@@ -35,12 +35,12 @@ FXDEFMAP(CommandWindow) CommandWindowMap[]={
 
 
 // Object implementation
-FXIMPLEMENT(CommandWindow,FXDialogBox,CommandWindowMap,ARRAYNUMBER(CommandWindowMap))
+FXIMPLEMENT(CommandWindow,DialogBox,CommandWindowMap,ARRAYNUMBER(CommandWindowMap))
 
 
 // Construct window which will always float over the owner window
 CommandWindow::CommandWindow(FXWindow *owner, const FXString& name, FXString strcmd, FXint nblines, FXint nbcols):
-        FXDialogBox(owner,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE,0,0,0,0, 6,6,6,6, 4,4)
+        DialogBox(owner,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE,0,0,0,0, 6,6,6,6, 4,4)
 {
 
     // Get command to execute
@@ -62,7 +62,7 @@ CommandWindow::CommandWindow(FXWindow *owner, const FXString& name, FXString str
 
 // Construct free-floating window
 CommandWindow::CommandWindow(FXApp* a, const FXString& name, FXString strcmd, FXint nblines, FXint nbcols):
-        FXDialogBox(a,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE|DECOR_MINIMIZE,0,0,0,0, 6,6,6,6, 4,4)
+        DialogBox(a,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE|DECOR_MINIMIZE,0,0,0,0, 6,6,6,6, 4,4)
 {
     // Get command to execute
     command=strcmd;
@@ -94,10 +94,11 @@ void CommandWindow::create()
 	if(!fontspec.empty())
 	{
     	FXFont* font=new FXFont(getApp(),fontspec);
+		font->create();
         text->setFont(font);
 	}
 
-    FXDialogBox::create();
+    DialogBox::create();
     show(PLACEMENT_SCREEN);
     
 	// Execute command
@@ -161,7 +162,7 @@ FXint CommandWindow::execCmd(FXString command)
         ::close(pipes[0]);            // Close the child's
         ::close(pipes[1]);            // pipes.
         args[0] = "sh";               // Setup arguments
-        args[1] = "-c";               // to run command
+        args[1] = "-vc";              // to run command (option -v to display the command to execute)
         args[2] = (char*)command.text();  // in a shell in
         args[3] = NULL;               // a new process.
         execvp(args[0], args);        // Start a new process which will execute the command.
@@ -221,13 +222,12 @@ long CommandWindow::onWatchProcess(FXObject*,FXSelector,void*)
                         break;
                 }
             }
-        }
+        }		
 	}
     
 	else
     {
         // Child has finished.
-
 		// Read data from the finished child
 		while ((nread = read(pipes[0], buf, sizeof(buf)-1)) > 0 )
 		{
