@@ -23,13 +23,20 @@ FXDEFMAP(DialogBox) DialogBoxMap[]={
 FXIMPLEMENT(DialogBox,FXTopWindow,DialogBoxMap,ARRAYNUMBER(DialogBoxMap))
 
 
-// Construct
+// Contruct dialog which will stay on top of owner
 DialogBox::DialogBox(FXWindow* win,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
         FXTopWindow(win,name,NULL,NULL,opts,x,y,w,h,pl,pr,pt,pb,hs,vs)
 {
     _option = 0;
 }
 
+// Contruct free floating dialog
+DialogBox::DialogBox(FXApp* a,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
+        FXTopWindow(a,name,NULL,NULL,opts,x,y,w,h,pl,pr,pt,pb,hs,vs)
+{
+    _option = 0;
+}
+
 
 // Close window & cancel out of dialog
 long DialogBox::onClose(FXObject*,FXSelector,void*)
@@ -133,28 +140,23 @@ void DialogBox::show(FXuint placement)
 // Keyboard press; handle escape and return to close the dialog
 long DialogBox::onKeyPress(FXObject* sender,FXSelector sel,void* ptr)
 {
-    FXEvent* event=(FXEvent*)ptr;
-    switch(event->code)
-    {
-    case KEY_Escape:
-        handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),NULL);
+	if (FXTopWindow::onKeyPress(sender,sel,ptr))
 		return 1;
-    case KEY_KP_Enter:
-    case KEY_Return:
-        handle(this,FXSEL(SEL_COMMAND,ID_ACCEPT),NULL);
+	if (((FXEvent*)ptr)->code==KEY_Escape)
+	{
+		handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),NULL);
 		return 1;
-    default:
-        FXTopWindow::onKeyPress(sender,sel,ptr);
-		return 1;
-    }
+	}
 	return 0;
 }
 
+
 // Execute dialog box modally
 FXuint DialogBox::execute(FXuint placement)
 {
     create();
     show(placement);
+	getApp()->refresh();
     return getApp()->runModalFor(this);
 }
 
