@@ -12,7 +12,7 @@
 #include "xfedefs.h"
 #include "icons.h"
 #include "DirList.h"
-#include "InputDialog.h"
+#include "BrowseInputDialog.h"
 #include "File.h"
 #include "FileDict.h"
 #include "MessageBox.h"
@@ -40,6 +40,7 @@ extern FXStringDict* mtdevices;
 extern FXStringDict* updevices;
 #endif
 
+extern FXbool allowPopupScroll;
 
 
 // Object implementation
@@ -50,7 +51,7 @@ FXIMPLEMENT(DirItem,FXTreeItem,NULL,0)
 // Map
 FXDEFMAP(DirList) DirListMap[]={
                                    FXMAPFUNC(SEL_DRAGGED,0,DirList::onDragged),
-                                   FXMAPFUNC(SEL_TIMEOUT,DirList::ID_REFRESH_TIMER,DirList::onRefreshTimer),
+                                   FXMAPFUNC(SEL_TIMEOUT,DirList::ID_REFRESH_TIMER,DirList::onCmdRefreshTimer),
 #if defined(linux)
                                    FXMAPFUNC(SEL_TIMEOUT,DirList::ID_MTDEVICES_REFRESH,DirList::onMtdevicesRefresh),
                                    FXMAPFUNC(SEL_TIMEOUT,DirList::ID_UPDEVICES_REFRESH,DirList::onUpdevicesRefresh),
@@ -76,6 +77,10 @@ FXDEFMAP(DirList) DirListMap[]={
                                    FXMAPFUNC(SEL_UPDATE,DirList::ID_SET_PATTERN,DirList::onUpdSetPattern),
                                    FXMAPFUNC(SEL_UPDATE,DirList::ID_SORT_REVERSE,DirList::onUpdSortReverse),
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_SHOW_HIDDEN,DirList::onCmdShowHidden),
+								   FXMAPFUNC(SEL_COMMAND,DirList::ID_DRAG_COPY,DirList::onCmdDragCopy),
+								   FXMAPFUNC(SEL_COMMAND,DirList::ID_DRAG_MOVE,DirList::onCmdDragMove),
+								   FXMAPFUNC(SEL_COMMAND,DirList::ID_DRAG_LINK,DirList::onCmdDragLink),
+								   FXMAPFUNC(SEL_COMMAND,DirList::ID_DRAG_REJECT,DirList::onCmdDragReject),
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_HIDE_HIDDEN,DirList::onCmdHideHidden),
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_TOGGLE_HIDDEN,DirList::onCmdToggleHidden),
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_SHOW_FILES,DirList::onCmdShowFiles),
@@ -83,6 +88,7 @@ FXDEFMAP(DirList) DirListMap[]={
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_TOGGLE_FILES,DirList::onCmdToggleFiles),
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_SET_PATTERN,DirList::onCmdSetPattern),
                                    FXMAPFUNC(SEL_COMMAND,DirList::ID_SORT_REVERSE,DirList::onCmdSortReverse),
+								   FXMAPFUNC(SEL_COMMAND,DirList::ID_REFRESH,DirList::onCmdRefresh),
                                };
 
 
@@ -366,6 +372,38 @@ long DirList::onDNDMotion(FXObject* sender,FXSelector sel,void* ptr)
 }
 
 
+// Set drag type to copy
+long DirList::onCmdDragCopy(FXObject* sender,FXSelector sel,void* ptr)
+{
+	dropaction=DRAG_COPY;
+	return 1;
+}
+
+
+// Set drag type to move
+long DirList::onCmdDragMove(FXObject* sender,FXSelector sel,void* ptr)
+{
+	dropaction=DRAG_MOVE;
+	return 1;
+}
+
+
+// Set drag type to symlink
+long DirList::onCmdDragLink(FXObject* sender,FXSelector sel,void* ptr)
+{
+	dropaction=DRAG_LINK;
+	return 1;
+}
+
+
+// Cancel drag action
+long DirList::onCmdDragReject(FXObject* sender,FXSelector sel,void* ptr)
+{
+	dropaction=DRAG_REJECT;
+	return 1;
+}
+
+
 // Handle drag-and-drop drop
 long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
 {
@@ -394,6 +432,23 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
     if(FXTreeList::onDNDDrop(sender,sel,ptr))
         return 1;
 
+	// Display a popup to select the drag type
+	dropaction=DRAG_REJECT;
+	FXMenuPane menu(this);
+    FXint x,y;
+    FXuint state;
+    getRoot()->getCursorPosition(x,y,state);
+   	new FXMenuCommand(&menu,_("Copy here"),copy_clpicon,this,DirList::ID_DRAG_COPY);
+   	new FXMenuCommand(&menu,_("Move here"),moveiticon,this,DirList::ID_DRAG_MOVE);
+   	new FXMenuCommand(&menu,_("Link here"),minilinkicon,this,DirList::ID_DRAG_LINK);
+	new FXMenuSeparator(&menu);
+   	new FXMenuCommand(&menu,_("Cancel"),NULL,this,DirList::ID_DRAG_REJECT);
+    menu.create();
+    allowPopupScroll=TRUE;  // Allow keyboard scrolling
+	menu.popup(NULL,x,y);
+	getApp()->runModalWhileShown(&menu);
+    allowPopupScroll=FALSE;
+	
     // Get uri-list of files being dropped
     if(getDNDData(FROM_DRAGNDROP,urilistType,data,len))
     {
@@ -417,27 +472,25 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
             f=new File(this,_("File copy"),COPY);
         else if (dropaction==DRAG_MOVE)
             f=new File(this,_("File move"),MOVE);
-        else if ((dropaction==DRAG_LINK) && (num==1))
-            f=new File(this,_("File symlink"),SYMLINK);
-        else if ((dropaction==DRAG_LINK) && (num>1))
-		{
-			MessageBox::error(this,BOX_OK,_("Error"),_("Can't symlink multiple selection!"));
-			return 0;
-		}
+        else if (dropaction==DRAG_LINK)
+			f=new File(this,_("File symlink"),SYMLINK);
 		else
             return 0;
 
+        // Target directory
+		FXString targetdir=dropdirectory;
+
         while(*p)
         {
             while(*q && *q!='\r')
                 q++;
             FXString url(p,q-p);
             FXString source(FXURL::fileFromURL(url));
-            FXString target(dropdirectory+PATHSEPSTRING+FXPath::name(source));
+            FXString target(targetdir+PATHSEPSTRING+FXPath::name(source));
             FXString sourcedir=FXPath::directory(source);
-            FXString targetdir=FXPath::directory(target);
 
-            // File operation dialog, if needed
+            /*
+			// File operation dialog, if needed
             if(ask_before_copy & showdialog)
             {
                 FXIcon *icon=NULL;
@@ -460,25 +513,30 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
                     else
                         message=_("Move ")+FXStringVal(num)+_(" files/folders.\nFrom: ")+sourcedir;
                 }
-                else if ((dropaction==DRAG_LINK) && (num==1))
+                else if (dropaction==DRAG_LINK)
                 {
                     title=_("Symlink ");
                     icon=link_bigicon;
-					message=title+source;
-                }
+                    if (num==1)
+                        message=title+source;
+                    else
+                        message=_("Symlink ")+FXStringVal(num)+_(" files/folders.\nFrom: ")+sourcedir;
+				}
 
-                InputDialog* dialog = new InputDialog(this,targetdir,message,title,_("To:"),icon);
+                // The target directory can be modified here
+                BrowseInputDialog* dialog = new BrowseInputDialog(this,targetdir,message,title,_("To:"),icon);
                 dialog->CursorEnd();
                 int rc=1;
                 rc=dialog->execute();
                 target=dialog->getText();
-                target=::filePath(target);
+                targetdir=::filePath(target);
                 if (num>1)
                     showdialog=FALSE;
                 delete dialog;
                 if (!rc)
                     return 0;
             }
+			*/
 
             // Move the source file
             if(dropaction==DRAG_MOVE)
@@ -975,7 +1033,7 @@ long DirList::onUpdevicesRefresh(FXObject*,FXSelector,void*)
     if(mtab)
     {
         while((mnt=getmntent(mtab)))
-        {
+        {			
 			// To fix an issue with some Linux distributions
 			if (!::streq(mnt->mnt_dir,"/dev/.static/dev"))
 			{
@@ -989,7 +1047,7 @@ long DirList::onUpdevicesRefresh(FXObject*,FXSelector,void*)
 				}
 				else
 					mtstate="up";
-
+				
 				if (updevices->find(mnt->mnt_dir))
 					updevices->remove(mnt->mnt_dir);
 				updevices->insert(mnt->mnt_dir,mtstate.text());
@@ -1020,7 +1078,7 @@ long DirList::onUpdevicesRefresh(FXObject*,FXSelector,void*)
 
 
 // Refresh with timer
-long DirList::onRefreshTimer(FXObject*,FXSelector,void*)
+long DirList::onCmdRefreshTimer(FXObject*,FXSelector,void*)
 {
     if(flags&FLAG_UPDATE)
     {
@@ -1033,13 +1091,15 @@ long DirList::onRefreshTimer(FXObject*,FXSelector,void*)
     return 0;
 }
 
+
 // Force refresh
-long DirList::onRefresh(FXObject*,FXSelector,void*)
+long DirList::onCmdRefresh(FXObject*,FXSelector,void*)
 {
     scan(TRUE);
     return 0;
 }
 
+
 // Scan items to see if listing is necessary
 void DirList::scan(FXbool force)
 {
