@@ -118,8 +118,8 @@ private:
     DirList(const DirList&);
     DirList &operator=(const DirList&);
 public:
-    long onRefresh(FXObject*,FXSelector,void*);
-    long onRefreshTimer(FXObject*,FXSelector,void*);
+    long onCmdRefresh(FXObject*,FXSelector,void*);
+    long onCmdRefreshTimer(FXObject*,FXSelector,void*);
 #if defined(linux)
     long onMtdevicesRefresh(FXObject*,FXSelector,void*);
     long onUpdevicesRefresh(FXObject*,FXSelector,void*);
@@ -155,6 +155,10 @@ public:
     long onUpdSortReverse(FXObject*,FXSelector,void*);
     long onCmdSortCase(FXObject*,FXSelector,void*);
     long onUpdSortCase(FXObject*,FXSelector,void*);
+	long onCmdDragCopy(FXObject* sender,FXSelector,void*);
+	long onCmdDragMove(FXObject* sender,FXSelector,void*);
+	long onCmdDragLink(FXObject* sender,FXSelector,void*);
+	long onCmdDragReject(FXObject* sender,FXSelector,void*);
 public:
     static FXint ascending(const TreeItem* a,const TreeItem* b);
     static FXint descending(const TreeItem* a,const TreeItem* b);
@@ -176,6 +180,11 @@ public:
         ID_UPDEVICES_REFRESH,
         ID_MTDEVICES_REFRESH,
 #endif
+		ID_DRAG_COPY,
+		ID_DRAG_MOVE,
+		ID_DRAG_LINK,
+		ID_DRAG_REJECT,
+		ID_REFRESH,
         ID_LAST
     };
 public:
