@@ -2,6 +2,9 @@
 #define DIRPANEL_H
 #include "DirList.h"
 #include "Properties.h"
+#include "InputDialog.h"
+#include "ArchInputDialog.h"
+#include "BrowseInputDialog.h"
 
 
 class DirPanel : public FXVerticalFrame
@@ -17,6 +20,15 @@ protected:
     FXLabel	*status;
 	FXString homelocation;
 	FXString trashlocation;
+	FXDragType urilistType;        // Standard uri-list type
+	FXDragType xfelistType;        // Xfe, Gnome and XFCE list type
+	FXDragType kdelistType;        // KDE list type
+	FXDragType utf8Type;           // UTF-8 text type
+	FXbool clipboard_locked;       // Clipboard locked to prevent changes when viewing it
+	InputDialog* newdirdialog;
+	ArchInputDialog* archdialog;
+	BrowseInputDialog* operationdialog;
+	FXbool paste;
 
 public:
 DirPanel(FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255),FXColor listforecolor=FXRGB(0,0,0),FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
@@ -25,7 +37,7 @@ DirPanel(FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255),FXColor listfor
     virtual void create();
     ~DirPanel();
     enum {
-        ID_DIR_PANEL=FXVerticalFrame::ID_LAST,
+        ID_FILELIST=FXVerticalFrame::ID_LAST,
 		ID_EXPANDTREE,
 		ID_TOGGLE_HIDDEN,
 		ID_COLLAPSEDIR,
@@ -33,6 +45,7 @@ DirPanel(FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255),FXColor listfor
 		ID_PROPERTIES,
 		ID_ARCHIVE,
 		ID_DIR_COPY,
+		ID_DIR_CUT,
 		ID_DIR_COPYTO,
 		ID_DIR_MOVE,
 		ID_DIR_RENAME,
@@ -41,9 +54,9 @@ DirPanel(FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255),FXColor listfor
 		ID_DIR_TRASH,
 		ID_NEW_DIR,
 		ID_XTERM,
-		ID_COPY_CLP,
-		ID_CUT_CLP,
-		ID_PASTE_CLP,
+		ID_COPY_CLIPBOARD,
+		ID_CUT_CLIPBOARD,
+		ID_PASTE_CLIPBOARD,
         ID_TOGGLE_TREE,
 		ID_TITLE,
 		ID_DIRSIZE_REFRESH,
@@ -56,6 +69,9 @@ DirPanel(FXComposite *p,FXColor listbackcolor=FXRGB(255,255,255),FXColor listfor
 	long exploreUp(DirItem *item, const DirItem *rootitem, const FXint task);
 	long exploreDown(DirItem *item, const DirItem *rootitem, const FXint task);
 public:
+	long onClipboardGained(FXObject*,FXSelector,void*);
+	long onClipboardLost(FXObject*,FXSelector,void*);
+	long onClipboardRequest(FXObject*,FXSelector,void*);
 	long onCmdToggleHidden(FXObject*,FXSelector,void*);
 	long onUpdToggleHidden(FXObject* sender,FXSelector,void*);
 	long onPopupMenu(FXObject*,FXSelector,void*);
@@ -68,7 +84,7 @@ public:
 	long onCmdDirTrash(FXObject*,FXSelector,void*);
 	long onCmdNewDir(FXObject*,FXSelector,void*);
 	long onCmdXTerm(FXObject*,FXSelector,void*);
-	long onCmdClp(FXObject*,FXSelector,void*);
+	long onCmdCopyCut(FXObject*,FXSelector,void*);
 	long onCmdPaste(FXObject*,FXSelector,void*);
 	long onUpdPaste(FXObject*,FXSelector,void*);
 	long onCmdDirectory(FXObject*,FXSelector,void*);
