@@ -29,6 +29,7 @@
 #include <fox-1.6/fx.h>
 #include <fox-1.6/fxkeys.h>
 #include <fox-1.6/FXPNGIcon.h>
+#include <fox-1.6/FXUTF8Codec.h>
 
 #include "xfedefs.h"
 #include "icons.h"
@@ -55,6 +56,52 @@ extern FXStringDict* updevices;
 #define PDIALOG_WIDTH 150
 
 
+// Test if a string (typically a file name) is encoded in UTF-8
+// Note that a pure ASCII string is considered here as UTF-8
+FXbool isUtf8(const FXchar* string, FXuint length)
+{
+	FXchar s[4];
+	const FXchar BOM[] = { 0xEF, 0xBB, 0xBF, '\0' };
+	
+	// Keep only length left bytes
+	FXString str=string;
+	str=str.left(length);
+	
+	// Convert forth and back to UTF8
+	FXUTF8Codec utf8;
+	FXString utf8str=utf8.mb2utf(utf8.utf2mb(str));
+ 
+	// Strings are equal => UTF8 
+	if (str==utf8str)
+		return TRUE;
+	
+	// Strings not equal => test if BOM is present
+	else
+	{
+		// String too small to contain BOM
+		if (length<=2)
+			return FALSE;
+		
+		// Test if string contains BOM
+		else
+		{
+			s[0]=string[0];
+			s[1]=string[1];
+			s[2]=string[2];
+			s[3]='\0';
+			
+			// String contains BOM => UTF8
+			if (::streq(s,BOM))
+				return TRUE;
+			
+			// String don't contain BOM
+			else
+				return FALSE;
+		}
+	}
+}
+
+
 // Replacement of the stat function
 int statrep(const char *filename, struct stat *buf)
 {
@@ -496,6 +543,9 @@ FXbool isGroupMember(gid_t gid)
 	return FALSE;
 }
 
+
+
+/*
 // Check if the file or the link refered file is writable
 // Doesn't use the standard FOX function because of the stat timeout 
 FXbool isWritable(const FXString& file)
@@ -545,9 +595,11 @@ FXbool isWritable(const FXString& file)
 		return FALSE;
 }
 
+
+
 // Check if the file or the link itself is writable
 // Doesn't use the standard FOX function because of the stat timeout 
-FXbool isLWritable(const FXString& file)
+FXbool isLinkWritable(const FXString& file)
 {
   	struct stat status;
 	FXuint uid=getuid();
@@ -595,7 +647,8 @@ FXbool isLWritable(const FXString& file)
 }
 
 
-// Check if the file or the link refered file is readable
+
+// Check if the file or link is readable
 // Doesn't use the standard FOX function because of the stat timeout 
 FXbool isReadable(const FXString& file)
 {
@@ -603,7 +656,7 @@ FXbool isReadable(const FXString& file)
 	FXuint uid=getuid();
 	
 	// File exists and can be stated
-	if (!file.empty() && (statrep(file.text(),&status)==0))
+	if (!file.empty() && (lstatrep(file.text(),&status)==0))
 	{
 		// We are root
 		if (uid==0)
@@ -694,6 +747,72 @@ FXbool isReadExecutable(const FXString& file)
 	else
 		return FALSE;
 }
+*/
+
+
+
+// Check if the file or the link refered file is readable AND executable
+// Function used to test if we can enter a directory
+// Uses the access() system function 
+FXbool isReadExecutable(const FXString& file)
+{	
+  	struct stat status;
+
+	// File exists and can be stated
+	if (!file.empty() && (statrep(file.text(),&status)==0))
+	{
+		int ret=access(file.text(),R_OK|X_OK);
+		if (ret==0)
+			return TRUE;
+		else
+			return FALSE;
+	}	
+	
+	// File doesn't exist
+	else
+		return FALSE;
+}
+
+
+FXbool isReadable(const FXString& file)
+{	
+  	struct stat status;
+
+	// File exists and can be stated
+	if (!file.empty() && (statrep(file.text(),&status)==0))
+	{
+		int ret=access(file.text(),R_OK);
+		if (ret==0)
+			return TRUE;
+		else
+			return FALSE;
+	}	
+	
+	// File doesn't exist
+	else
+		return FALSE;
+}
+
+
+FXbool isWritable(const FXString& file)
+{	
+  	struct stat status;
+
+	// File exists and can be stated
+	if (!file.empty() && (statrep(file.text(),&status)==0))
+	{
+		int ret=access(file.text(),W_OK);
+		if (ret==0)
+			return TRUE;
+		else
+			return FALSE;
+	}	
+	
+	// File doesn't exist
+	else
+		return FALSE;
+}
+
 
 // Check if file represents a link
 FXbool isLink(const FXString& file)
@@ -702,7 +821,7 @@ FXbool isLink(const FXString& file)
   	return !file.empty() && (lstatrep(file.text(),&status)==0) && S_ISLNK(status.st_mode);
 }
 
-// Get file info
+// Get file info (file or link refered file)
 FXbool info(const FXString& file,struct stat& inf)
 {
   	return !file.empty() && (statrep(file.text(),&inf)==0);
@@ -759,10 +878,10 @@ FXDEFMAP(File) FileMap[]={
                          };
 
 // Object implementation
-FXIMPLEMENT(File,FXDialogBox,FileMap,ARRAYNUMBER(FileMap))
+FXIMPLEMENT(File,DialogBox,FileMap,ARRAYNUMBER(FileMap))
 
 // Construct object
-File::File(FXWindow *owner, FXString title, const unsigned int operation):FXDialogBox(owner,title,DECOR_TITLE|DECOR_BORDER)
+File::File(FXWindow *owner, FXString title, const unsigned int operation):DialogBox(owner,title,DECOR_TITLE|DECOR_BORDER)
 {
     // Progress window
 	FXPacker *buttons=new FXPacker(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,10,10,PDIALOG_WIDTH,PDIALOG_WIDTH,5,5);
@@ -878,7 +997,7 @@ File::~File()
 // Create and initialize
 void File::create()
 {
-	FXDialogBox::create();
+	DialogBox::create();
 }
 
 // Force check of timeout for progress dialog (to avoid latency problems)
@@ -1136,7 +1255,7 @@ FXbool File::copydir(const FXString& source,const FXString& target,struct stat&
 FXbool File::copyrec(const FXString& source,const FXString& target,inodelist* inodes, const FXbool preserve_date)
 {
     struct stat status1,status2;
-
+	
     // Source file or directory does not exist
     if(lstatrep(source.text(),&status1)!=0)
         return FALSE;
@@ -1554,13 +1673,20 @@ FXbool File::symlink(const FXString& source,const FXString& target)
 		return FALSE;
 	}
 
+	// Target is an existing directory
+	FXString targetfile;
+	if (::isDirectory(target))
+		targetfile=target+PATHSEPSTRING+FXPath::name(source);
+	else
+		targetfile=target;
+
 	// Target already exists
-	if(::exists(target))
+	if(::exists(targetfile))
 	{
 		// Overwrite dialog if necessary
 		if (!(overwrite_all | skip_all))
     	{
-			FXString msg=_("File ")+target+_(" already exists. Overwrite?");
+			FXString msg=_("File ")+targetfile+_(" already exists. Overwrite?");
        		ConfirmOverWriteBox* dlg = new ConfirmOverWriteBox(this,_("Confirm Overwrite"),msg);
        		FXuint answer=dlg->execute();
 			delete dlg;
@@ -1585,7 +1711,7 @@ FXbool File::symlink(const FXString& source,const FXString& target)
 	}
     
 	// Create symbolic link using the standard C function
-	FXint ret=::symlink(source.text(),target.text());	
+	FXint ret=::symlink(source.text(),targetfile.text());	
 	if (ret==0)
 		return TRUE;
 	else
