@@ -12,7 +12,10 @@
 #include <dirent.h>
 #include <fox-1.6/fx.h>
 
+#include "DialogBox.h"
 
+
+FXbool isUtf8(const FXchar* string, FXuint length);
 int statrep(const char *filename, struct stat *buf);
 int lstatrep(const char *filename, struct stat *buf);
 #ifdef linux
@@ -36,7 +39,7 @@ FXbool isDirectory(const FXString& file);
 FXbool isFile(const FXString& file);
 FXbool isGroupMember(gid_t gid);
 FXbool isWritable(const FXString& file);
-FXbool isLWritable(const FXString& file);
+//FXbool isLinkWritable(const FXString& file);
 FXbool isReadable(const FXString& file);
 FXbool isReadExecutable(const FXString& file);
 FXbool isLink(const FXString& file);
@@ -72,7 +75,7 @@ struct inodelist
     inodelist *next;
 };
 
-class File : public FXDialogBox
+class File : public DialogBox
 {
     FXDECLARE(File)
 protected:
@@ -101,7 +104,7 @@ public:
 
 	enum 
 	{
-		ID_CANCEL_BUTTON=FXDialogBox::ID_LAST,
+		ID_CANCEL_BUTTON=DialogBox::ID_LAST,
 		ID_TIMEOUT,
 		ID_LAST
 	};
