@@ -6,10 +6,12 @@
 #include <fox-1.6/fxkeys.h>
 #include <fox-1.6/FXPNGIcon.h>
 
+#include "xfedefs.h"
 #include "File.h"
 #include "FileList.h"
 #include "DirBox.h"
 #include "FileDialog.h"
+#include "InputDialog.h"
 #include "DirHistBox.h"
 #include "MessageBox.h"
 #include "icons.h"
@@ -20,6 +22,8 @@
 extern FXbool single_click_diropen;
 extern FXbool single_click_fileopen;
 
+extern FXbool allowPopupScroll;
+
 
 
 // Map
@@ -42,7 +46,9 @@ FXDEFMAP(FileSelector) FileSelectorMap[]={
 			FXMAPFUNC(SEL_UPDATE,FileSelector::ID_DIR_FORWARD_HIST,FileSelector::onUpdDirForwardHist),
             FXMAPFUNC(SEL_COMMAND,FileSelector::ID_DIR_TREE,FileSelector::onCmdDirTree),
             FXMAPFUNC(SEL_COMMAND,FileSelector::ID_HOME,FileSelector::onCmdHome),
+            FXMAPFUNC(SEL_COMMAND,FileSelector::ID_NEWDIR,FileSelector::onCmdNewDir),
             FXMAPFUNC(SEL_COMMAND,FileSelector::ID_WORK,FileSelector::onCmdWork),
+            FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,FileSelector::ID_FILELIST,FileSelector::onCmdPopupMenu),
         };
 
 
@@ -75,8 +81,8 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
     readonly=new FXCheckButton(filterframe,_("Read Only"),NULL,0,ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_CENTER_Y);
     cancel=new FXButton(fields,_("&Cancel"),NULL,NULL,0,BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,0,0,0,0,20,20);
     FXHorizontalFrame* frame=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0);
-    filebox=new FileList(frame,this,ID_FILELIST,ICONLIST_MINI_ICONS|ICONLIST_BROWSESELECT|ICONLIST_AUTOSIZE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    new FXLabel(buttons,_("Folder:"),NULL,LAYOUT_CENTER_Y);
+    list=new FileList(frame,this,ID_FILELIST,ICONLIST_MINI_ICONS|ICONLIST_BROWSESELECT|ICONLIST_AUTOSIZE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	new FXLabel(buttons,_("Folder:"),NULL,LAYOUT_CENTER_Y);
     dirbox=new DirBox(buttons,this,ID_DIR_TREE,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_CENTER_Y,0,0,160,0);
 	dirbox->setNumVisible(5);
     new FXFrame(buttons,LAYOUT_FIX_WIDTH,0,0,4,1);
@@ -87,11 +93,12 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
     new FXButton(buttons,_("\tGo up one folder\tMove up to higher folder."),dirupicon,this,ID_DIR_UP,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
     new FXButton(buttons,_("\tGo to home folder\tBack to home folder."),homeicon,this,ID_HOME,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
     new FXButton(buttons,_("\tGo to working folder\tBack to working folder."),workicon,this,ID_WORK,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow list\tDisplay folder with small icons."),smalliconsicon,filebox,FileList::ID_SHOW_MINI_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow icons\tDisplay folder with big icons."),bigiconsicon,filebox,FileList::ID_SHOW_BIG_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXButton(buttons,_("\tShow details\tDisplay detailed folder listing."),detailsicon,filebox,FileList::ID_SHOW_DETAILS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXToggleButton(buttons,_("\tShow hidden files\tShow hidden files and folders."),_("\tHide hidden Files\tHide hidden files and folders."),hidehiddenicon,showhiddenicon,filebox,FileList::ID_TOGGLE_HIDDEN,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
-    new FXToggleButton(buttons,_("\tShow thumbnails"),_("\tHide thumbnails"),showthumbicon,hidethumbicon,filebox,FileList::ID_TOGGLE_THUMBNAILS,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,_("\tNew folder\tCreate new folder."),newfoldericon,this,ID_NEWDIR,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,_("\tShow list\tDisplay folder with small icons."),smalliconsicon,list,FileList::ID_SHOW_MINI_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,_("\tShow icons\tDisplay folder with big icons."),bigiconsicon,list,FileList::ID_SHOW_BIG_ICONS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXButton(buttons,_("\tShow details\tDisplay detailed folder listing."),detailsicon,list,FileList::ID_SHOW_DETAILS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXToggleButton(buttons,_("\tShow hidden files\tShow hidden files and folders."),_("\tHide hidden Files\tHide hidden files and folders."),showhiddenicon,hidehiddenicon,list,FileList::ID_TOGGLE_HIDDEN,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
+    new FXToggleButton(buttons,_("\tShow thumbnails"),_("\tHide thumbnails"),showthumbicon,hidethumbicon,list,FileList::ID_TOGGLE_THUMBNAILS,TOGGLEBUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0, 3,3,3,3);
     readonly->hide();
   	if(table)
 	{
@@ -100,18 +107,18 @@ FileSelector::FileSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint op
     	table->addAccel(MKUINT(KEY_BackSpace,0),this,FXSEL(SEL_COMMAND,ID_DIR_UP));
     	table->addAccel(MKUINT(KEY_h,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_HOME));
     	table->addAccel(MKUINT(KEY_w,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_WORK));
-    	table->addAccel(MKUINT(KEY_a,CONTROLMASK),filebox,FXSEL(SEL_COMMAND,FileList::ID_SELECT_ALL));
-    	table->addAccel(MKUINT(KEY_i,CONTROLMASK),filebox,FXSEL(SEL_COMMAND,FileList::ID_DESELECT_ALL));
+    	table->addAccel(MKUINT(KEY_a,CONTROLMASK),list,FXSEL(SEL_COMMAND,FileList::ID_SELECT_ALL));
+    	table->addAccel(MKUINT(KEY_i,CONTROLMASK),list,FXSEL(SEL_COMMAND,FileList::ID_DESELECT_ALL));
     }
-    setSelectMode(SELECTFILE_ANY);    // For backward compatibility, this HAS to be the default!
+    setSelectMode(SELECT_FILE_ANY);    // For backward compatibility, this HAS to be the default!
     setPatternList(allfiles);
     setDirectory(FXSystem::getCurrentDirectory());
-    filebox->setFocus();
+    list->setFocus();
     accept->hide();
 
 	// Change default cursor if single click navigation
 	if (single_click_diropen & single_click_fileopen)
-		filebox->setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
+		list->setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
 }
 
 
@@ -123,9 +130,9 @@ long FileSelector::onCmdItemDoubleClicked(FXObject*,FXSelector,void* ptr)
         return 1;
 
     // If directory, open the directory
-    if(filebox->isItemDirectory(index))
+    if(list->isItemDirectory(index))
     {
-		FXString filepath=filebox->getItemPathname(index);
+		FXString filepath=list->getItemPathname(index);
 
 		// Does not have access
 		if(!::isReadExecutable(filepath))
@@ -139,9 +146,9 @@ long FileSelector::onCmdItemDoubleClicked(FXObject*,FXSelector,void* ptr)
     }
 
     // Only return if we wanted a file
-    if(selectmode!=SELECTFILE_DIRECTORY)
+    if((selectmode!=SELECT_FILE_DIRECTORY) && (selectmode!=SELECT_FILE_MIXED))
     {
-        if(filebox->isItemFile(index))
+        if(list->isItemFile(index))
         {
             FXObject *tgt=accept->getTarget();
             FXSelector sel=accept->getSelector();
@@ -162,27 +169,40 @@ long FileSelector::onCmdItemClicked(FXObject*,FXSelector,void* ptr)
 		if(index<0)
 			return 1;
 
-		// If directory, open the directory
-		if(single_click_diropen && filebox->isItemDirectory(index))
-		{
-			FXString filepath=filebox->getItemPathname(index);
+		// Test if a key modifier was pressed
+		FXint x, y;
+		FXuint state;
+		getCursorPosition(x,y,state);
 
-			// Does not have access
-			if(!::isReadExecutable(filepath))
-			{
-				MessageBox::error(this,BOX_OK,_("Error"),_(" Permission to: %s denied."), filepath.text());
-				return 0;
-			}
-			
-			setDirectory(filepath);
+		// Single click with control or shift
+		if (state&(CONTROLMASK|SHIFTMASK))
 			return 1;
-		}
-		else if(single_click_fileopen && filebox->isItemFile(index))
+		
+		// Single click without control or shift
+		else
 		{
-			FXObject *tgt=accept->getTarget();
-			FXSelector sel=accept->getSelector();
-			if(tgt)
-				tgt->handle(accept,FXSEL(SEL_COMMAND,sel),(void*)1);
+			// If directory, open the directory
+			if(single_click_diropen && list->isItemDirectory(index))
+			{
+				FXString filepath=list->getItemPathname(index);
+
+				// Does not have access
+				if(!::isReadExecutable(filepath))
+				{
+					MessageBox::error(this,BOX_OK,_("Error"),_(" Permission to: %s denied."), filepath.text());
+					return 0;
+				}
+				
+				setDirectory(filepath);
+				return 1;
+			}
+			else if(single_click_fileopen && list->isItemFile(index))
+			{
+				FXObject *tgt=accept->getTarget();
+				FXSelector sel=accept->getSelector();
+				if(tgt)
+					tgt->handle(accept,FXSEL(SEL_COMMAND,sel),(void*)1);
+			}
 		}
 	}
     return 1;
@@ -193,45 +213,51 @@ long FileSelector::onCmdItemSelected(FXObject*,FXSelector,void* ptr)
 {
     FXint index=(FXint)(long)ptr;
     FXString text,file;
-    if(selectmode==SELECTFILE_MULTIPLE)
+    if(selectmode==SELECT_FILE_MULTIPLE)
     {
-        for(FXint i=0; i<filebox->getNumItems(); i++)
+        for(FXint i=0; i<list->getNumItems(); i++)
         {
-            if(filebox->isItemFile(i) && filebox->isItemSelected(i))
+            if(list->isItemFile(i) && list->isItemSelected(i))
             {
                 if(!text.empty())
                     text+=' ';
-                text+="\""+filebox->getItemFilename(i)+"\"";
+                text+="\""+list->getItemFilename(i)+"\"";
             }
         }
         filename->setText(text);
     }
-    else if(selectmode==SELECTFILE_MULTIPLE_ALL)
+    else if(selectmode==SELECT_FILE_MULTIPLE_ALL)
     {
-        for(FXint i=0; i<filebox->getNumItems(); i++)
+        for(FXint i=0; i<list->getNumItems(); i++)
         {
-            if(filebox->isItemSelected(i) && filebox->getItemFilename(i)!="..")
+            if(list->isItemSelected(i) && list->getItemFilename(i)!="..")
             {
                 if(!text.empty())
                     text+=' ';
-                text+="\""+filebox->getItemFilename(i)+"\"";
+                text+="\""+list->getItemFilename(i)+"\"";
             }
         }
         filename->setText(text);
     }
-    else if(selectmode==SELECTFILE_DIRECTORY)
+    else if(selectmode==SELECT_FILE_DIRECTORY)
     {
-        if(filebox->isItemDirectory(index))
+        if(list->isItemDirectory(index))
         {
-            text=filebox->getItemFilename(index);
+            text=list->getItemFilename(index);
             filename->setText(text);
         }
     }
+    // Mode added to select both directories and files
+	else if(selectmode==SELECT_FILE_MIXED)
+    {
+        text=list->getItemFilename(index);
+        filename->setText(text);
+    }
     else
     {
-        if(filebox->isItemFile(index))
+        if(list->isItemFile(index))
         {
-            text=filebox->getItemFilename(index);
+            text=list->getItemFilename(index);
             filename->setText(text);
         }
     }
@@ -243,28 +269,28 @@ long FileSelector::onCmdItemSelected(FXObject*,FXSelector,void* ptr)
 long FileSelector::onCmdItemDeselected(FXObject*,FXSelector,void*)
 {
     FXString text,file;
-    if(selectmode==SELECTFILE_MULTIPLE)
+    if(selectmode==SELECT_FILE_MULTIPLE)
     {
-        for(FXint i=0; i<filebox->getNumItems(); i++)
+        for(FXint i=0; i<list->getNumItems(); i++)
         {
-            if(filebox->isItemFile(i) && filebox->isItemSelected(i))
+            if(list->isItemFile(i) && list->isItemSelected(i))
             {
                 if(!text.empty())
                     text+=' ';
-                text+="\""+filebox->getItemFilename(i)+"\"";
+                text+="\""+list->getItemFilename(i)+"\"";
             }
         }
         filename->setText(text);
     }
-    else if(selectmode==SELECTFILE_MULTIPLE_ALL)
+    else if(selectmode==SELECT_FILE_MULTIPLE_ALL)
     {
-        for(FXint i=0; i<filebox->getNumItems(); i++)
+        for(FXint i=0; i<list->getNumItems(); i++)
         {
-            if(filebox->isItemSelected(i) && filebox->getItemFilename(i)!="..")
+            if(list->isItemSelected(i) && list->getItemFilename(i)!="..")
             {
                 if(!text.empty())
                     text+=' ';
-                text+="\""+filebox->getItemFilename(i)+"\"";
+                text+="\""+list->getItemFilename(i)+"\"";
             }
         }
         filename->setText(text);
@@ -291,7 +317,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
         {
 
             // In directory mode:- we got our answer!
-            if(selectmode==SELECTFILE_DIRECTORY || selectmode==SELECTFILE_MULTIPLE_ALL)
+            if(selectmode==SELECT_FILE_DIRECTORY || selectmode==SELECT_FILE_MULTIPLE_ALL || selectmode==SELECT_FILE_MIXED)
             {
                 if(tgt)
                     tgt->handle(accept,FXSEL(SEL_COMMAND,sel),(void*)1);
@@ -300,7 +326,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
 
             // Hop over to that directory
             dirbox->setDirectory(path);
-            filebox->setDirectory(path);
+            list->setDirectory(path);
             filename->setText(FXString::null);
             return 1;
         }
@@ -313,7 +339,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
         {
 
             // In any mode, existing directory part is good enough
-            if(selectmode==SELECTFILE_ANY)
+            if(selectmode==SELECT_FILE_ANY)
             {
                 if(tgt)
                     tgt->handle(accept,FXSEL(SEL_COMMAND,sel),(void*)1);
@@ -321,7 +347,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
             }
 
             // In existing mode, the whole filename must exist and be a file
-            else if(selectmode==SELECTFILE_EXISTING)
+            else if(selectmode==SELECT_FILE_EXISTING)
             {
                 if(::isFile(path))
                 {
@@ -332,11 +358,11 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
             }
 
             // In multiple mode, return if all selected files exist
-            else if(selectmode==SELECTFILE_MULTIPLE)
+            else if(selectmode==SELECT_FILE_MULTIPLE)
             {
-                for(FXint i=0; i<filebox->getNumItems(); i++)
+                for(FXint i=0; i<list->getNumItems(); i++)
                 {
-                    if(filebox->isItemSelected(i) && filebox->isItemFile(i))
+                    if(list->isItemSelected(i) && list->isItemFile(i))
                     {
                         if(tgt)
                             tgt->handle(accept,FXSEL(SEL_COMMAND,sel),(void*)1);
@@ -348,9 +374,9 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
             // Multiple files and/or directories
             else
             {
-                for(FXint i=0; i<filebox->getNumItems(); i++)
+                for(FXint i=0; i<list->getNumItems(); i++)
                 {
-                    if(filebox->isItemSelected(i) && filebox->getItemFilename(i)!="..")
+                    if(list->isItemSelected(i) && list->getItemFilename(i)!="..")
                     {
                         if(tgt)
                             tgt->handle(accept,FXSEL(SEL_COMMAND,sel),(void*)1);
@@ -366,7 +392,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
 
         // Switch as far as we could go
         dirbox->setDirectory(dir);
-        filebox->setDirectory(dir);
+        list->setDirectory(dir);
 
         // Put the tail end back for further editing
         if(ISPATHSEP(path[dir.length()]))
@@ -386,7 +412,7 @@ long FileSelector::onCmdAccept(FXObject*,FXSelector,void*)
 // User clicked up directory button
 long FileSelector::onCmdDirUp(FXObject*,FXSelector,void*)
 {
-    setDirectory(FXPath::upLevel(filebox->getDirectory()));
+    setDirectory(FXPath::upLevel(list->getDirectory()));
     return 1;
 }
 
@@ -394,7 +420,7 @@ long FileSelector::onCmdDirUp(FXObject*,FXSelector,void*)
 // Can we still go up
 long FileSelector::onUpdDirUp(FXObject* sender,FXSelector,void*)
 {
-    if(FXPath::isTopDirectory(filebox->getDirectory()))
+    if(FXPath::isTopDirectory(list->getDirectory()))
         sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
     else
         sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
@@ -410,8 +436,8 @@ long  FileSelector::onCmdDirBack(FXObject*,FXSelector s,void* p)
 	FXString pathname;
 
 	// Get the filelist history
-	backhist=filebox->backhist;
-	forwardhist=filebox->forwardhist;
+	backhist=list->backhist;
+	forwardhist=list->forwardhist;
 
 	// Get the previous directory
 	item=backhist->getFirst();
@@ -420,10 +446,10 @@ long  FileSelector::onCmdDirBack(FXObject*,FXSelector s,void* p)
 	
 	// Update the history
 	backhist->removeFirstItem();
-	forwardhist->insertFirstItem(filebox->getDirectory());
+	forwardhist->insertFirstItem(list->getDirectory());
 	
 	// Go to to the previous directory
-	filebox->setDirectory(pathname,FALSE);
+	list->setDirectory(pathname,FALSE);
 	dirbox->setDirectory(pathname);
 
     return 1;
@@ -438,7 +464,7 @@ long  FileSelector::onUpdDirBack(FXObject* sender, FXSelector sel, void* ptr)
 	FXString pathname;
 
 	// Get the filelist history
-	backhist=filebox->backhist;
+	backhist=list->backhist;
 	
 	// Gray out the button if no item in history 
 	FXint num=backhist->getNumItems();
@@ -460,8 +486,8 @@ long  FileSelector::onCmdDirForward(FXObject*,FXSelector s,void* p)
 	FXString pathname;
 
 	// Get the filelist history
-	backhist=filebox->backhist;
-	forwardhist=filebox->forwardhist;
+	backhist=list->backhist;
+	forwardhist=list->forwardhist;
 
 	// Get the next directory
 	item=forwardhist->getFirst();
@@ -470,10 +496,10 @@ long  FileSelector::onCmdDirForward(FXObject*,FXSelector s,void* p)
 	
 	// Update the history
 	forwardhist->removeFirstItem();
-	backhist->insertFirstItem(filebox->getDirectory());
+	backhist->insertFirstItem(list->getDirectory());
 	
 	// Go to to the previous directory
-	filebox->setDirectory(pathname,FALSE);
+	list->setDirectory(pathname,FALSE);
 	dirbox->setDirectory(pathname);
 
     return 1;
@@ -488,7 +514,7 @@ long  FileSelector::onUpdDirForward(FXObject* sender, FXSelector sel, void* ptr)
 	FXString pathname;
 
 	// Get the filelist history
-	forwardhist=filebox->forwardhist;
+	forwardhist=list->forwardhist;
 	
 	// Gray out the button if no item in history 
 	FXint num=forwardhist->getNumItems();
@@ -509,8 +535,8 @@ long  FileSelector::onCmdDirBackHist(FXObject *sender,FXSelector sel,void* ptr)
 	FXString pathname;
 
 	// Get the filelist history
-	backhist=filebox->backhist;
-	forwardhist=filebox->forwardhist;
+	backhist=list->backhist;
+	forwardhist=list->forwardhist;
 
 	// Get all string items and display them in a list box
 	FXint num=backhist->getNumItems();
@@ -550,7 +576,7 @@ long  FileSelector::onCmdDirBackHist(FXObject *sender,FXSelector sel,void* ptr)
 			}
 			
 			// Update forward history
-			forwardhist->insertFirstItem(filebox->getDirectory());
+			forwardhist->insertFirstItem(list->getDirectory());
 			if (pos>0)
 			{
 				for (FXint i=0; i<=pos-1; i++)
@@ -559,7 +585,7 @@ long  FileSelector::onCmdDirBackHist(FXObject *sender,FXSelector sel,void* ptr)
 	
 			// Go to to the selected directory
 			pathname=dirs[pos];
-			filebox->setDirectory(pathname,FALSE);
+			list->setDirectory(pathname,FALSE);
 			dirbox->setDirectory(pathname);
 		}
 		delete[]dirs;
@@ -577,7 +603,7 @@ long  FileSelector::onUpdDirBackHist(FXObject* sender, FXSelector sel, void* ptr
 	FXString pathname;
 
 	// Get the filelist history
-	backhist=filebox->backhist;
+	backhist=list->backhist;
 	
 	// Gray out the button if no item in history 
 	FXint num=backhist->getNumItems();
@@ -598,8 +624,8 @@ long  FileSelector::onCmdDirForwardHist(FXObject *sender,FXSelector sel,void* pt
 	FXString pathname;
 
 	// Get the filelist history
-	backhist=filebox->backhist;
-	forwardhist=filebox->forwardhist;
+	backhist=list->backhist;
+	forwardhist=list->forwardhist;
 
 	// Get all string items and display them in a list box
 	FXint num=forwardhist->getNumItems();
@@ -638,7 +664,7 @@ long  FileSelector::onCmdDirForwardHist(FXObject *sender,FXSelector sel,void* pt
 			}
 			
 			// Update back history
-			backhist->insertFirstItem(filebox->getDirectory());
+			backhist->insertFirstItem(list->getDirectory());
 			if (pos>0)
 			{
 				for (FXint i=0; i<=pos-1; i++)
@@ -647,7 +673,7 @@ long  FileSelector::onCmdDirForwardHist(FXObject *sender,FXSelector sel,void* pt
 	
 			// Go to to the selected directory
 			pathname=dirs[pos];
-			filebox->setDirectory(pathname,FALSE);
+			list->setDirectory(pathname,FALSE);
 			dirbox->setDirectory(pathname);
 		}
 		delete[]dirs;
@@ -665,7 +691,7 @@ long  FileSelector::onUpdDirForwardHist(FXObject* sender, FXSelector sel, void*
 	FXString pathname;
 
 	// Get the filelist history
-	forwardhist=filebox->forwardhist;
+	forwardhist=list->forwardhist;
 	
 	// Gray out the button if no item in history 
 	FXint num=forwardhist->getNumItems();
@@ -685,6 +711,42 @@ long FileSelector::onCmdHome(FXObject*,FXSelector,void*)
     return 1;
 }
 
+// Create new directory
+long FileSelector::onCmdNewDir(FXObject*,FXSelector,void*)
+{
+	// Current item
+	list->setFocus();
+
+    FXString dirname="";	
+    FXString dirpath=list->getDirectory();
+    if(dirpath!=ROOTDIR)
+        dirpath+=PATHSEPSTRING;
+
+    InputDialog* dialog=new InputDialog(this,dirname,_("Create new folder..."),_("New Folder"));
+    dialog->CursorEnd();
+    if(dialog->execute(PLACEMENT_CURSOR))
+    {
+		dirname=dirpath+dialog->getText();
+        if(dirname!=dirpath)
+        {
+			// Create the new dir according to the current umask
+			int mask;
+			mask=umask(0);
+			umask(mask);
+			// note that the umask value is in decimal (511 means octal 0777)
+			FXint ret=::mkdir(dirname.text(),511 & ~mask);
+			if (ret==-1)
+			{
+         		MessageBox::error(this,BOX_OK_SU,_("Error"),"%s",strerror(errno));
+    			return 0;
+			}
+		}							
+    }
+    delete dialog;
+	
+    return 1;
+}
+
 
 // Back to current working directory
 long FileSelector::onCmdWork(FXObject*,FXSelector,void*)
@@ -698,8 +760,8 @@ long FileSelector::onCmdWork(FXObject*,FXSelector,void*)
 // Switched directories using directory tree
 long FileSelector::onCmdDirTree(FXObject*,FXSelector,void* ptr)
 {
-    filebox->setDirectory((FXchar*)ptr);
-    if(selectmode==SELECTFILE_DIRECTORY)
+    list->setDirectory((FXchar*)ptr);
+    if(selectmode==SELECT_FILE_DIRECTORY || selectmode==SELECT_FILE_MIXED)
         filename->setText(FXString::null);
     return 1;
 }
@@ -749,8 +811,8 @@ FXString FileSelector::extensionFromPattern(const FXString& pattern)
 long FileSelector::onCmdFilter(FXObject*,FXSelector,void* ptr)
 {
     FXString pat=patternFromText((FXchar*)ptr);
-    filebox->setPattern(pat);
-    if(selectmode==SELECTFILE_ANY)
+    list->setPattern(pat);
+    if(selectmode==SELECT_FILE_ANY)
     {
         FXString ext=extensionFromPattern(pat);
         if(!ext.empty())
@@ -768,9 +830,9 @@ long FileSelector::onCmdFilter(FXObject*,FXSelector,void* ptr)
 void FileSelector::setDirectory(const FXString& path)
 {
     FXString abspath=FXPath::absolute(path);
-    filebox->setDirectory(abspath);
+    list->setDirectory(abspath);
     dirbox->setDirectory(abspath);
-    if(selectmode!=SELECTFILE_ANY)
+    if(selectmode!=SELECT_FILE_ANY)
         filename->setText(FXString::null);
 }
 
@@ -778,7 +840,7 @@ void FileSelector::setDirectory(const FXString& path)
 // Get directory
 FXString FileSelector::getDirectory() const
 {
-    return filebox->getDirectory();
+    return list->getDirectory();
 }
 
 
@@ -786,7 +848,7 @@ FXString FileSelector::getDirectory() const
 void FileSelector::setFilename(const FXString& path)
 {
     FXString abspath=FXPath::absolute(path);
-    filebox->setCurrentFile(abspath);
+    list->setCurrentFile(abspath);
     dirbox->setDirectory(FXPath::directory(abspath));
     filename->setText(FXPath::name(abspath));
 }
@@ -796,26 +858,26 @@ void FileSelector::setFilename(const FXString& path)
 FXString FileSelector::getFilename() const
 {
     register FXint i;
-    if(selectmode==SELECTFILE_MULTIPLE_ALL)
+    if(selectmode==SELECT_FILE_MULTIPLE_ALL)
     {
-        for(i=0; i<filebox->getNumItems(); i++)
+        for(i=0; i<list->getNumItems(); i++)
         {
-            if(filebox->isItemSelected(i) && filebox->getItemFilename(i)!="..")
-                return FXPath::absolute(filebox->getDirectory(),filebox->getItemFilename(i));
+            if(list->isItemSelected(i) && list->getItemFilename(i)!="..")
+                return FXPath::absolute(list->getDirectory(),list->getItemFilename(i));
         }
     }
-    else if(selectmode==SELECTFILE_MULTIPLE)
+    else if(selectmode==SELECT_FILE_MULTIPLE)
     {
-        for(i=0; i<filebox->getNumItems(); i++)
+        for(i=0; i<list->getNumItems(); i++)
         {
-            if(filebox->isItemSelected(i) && filebox->isItemFile(i))
-                return FXPath::absolute(filebox->getDirectory(),filebox->getItemFilename(i));
+            if(list->isItemSelected(i) && list->isItemFile(i))
+                return FXPath::absolute(list->getDirectory(),list->getItemFilename(i));
         }
     }
     else
     {
         if(!filename->getText().empty())
-            return FXPath::absolute(filebox->getDirectory(),filename->getText());
+            return FXPath::absolute(list->getDirectory(),filename->getText());
     }
     return FXString::null;
 }
@@ -826,40 +888,40 @@ FXString* FileSelector::getFilenames() const
 {
     register FXString *files=NULL;
     register FXint i,n;
-    if(filebox->getNumItems())
+    if(list->getNumItems())
     {
-        if(selectmode==SELECTFILE_MULTIPLE_ALL)
+        if(selectmode==SELECT_FILE_MULTIPLE_ALL)
         {
-            for(i=n=0; i<filebox->getNumItems(); i++)
+            for(i=n=0; i<list->getNumItems(); i++)
             {
-                if(filebox->isItemSelected(i) && filebox->getItemFilename(i)!="..")
+                if(list->isItemSelected(i) && list->getItemFilename(i)!="..")
                     n++;
             }
             if(n)
             {
                 files=new FXString [n+1];
-                for(i=n=0; i<filebox->getNumItems(); i++)
+                for(i=n=0; i<list->getNumItems(); i++)
                 {
-                    if(filebox->isItemSelected(i) && filebox->getItemFilename(i)!="..")
-                        files[n++]=filebox->getItemPathname(i);
+                    if(list->isItemSelected(i) && list->getItemFilename(i)!="..")
+                        files[n++]=list->getItemPathname(i);
                 }
                 files[n]=FXString::null;
             }
         }
         else
         {
-            for(i=n=0; i<filebox->getNumItems(); i++)
+            for(i=n=0; i<list->getNumItems(); i++)
             {
-                if(filebox->isItemSelected(i) && filebox->isItemFile(i))
+                if(list->isItemSelected(i) && list->isItemFile(i))
                     n++;
             }
             if(n)
             {
                 files=new FXString [n+1];
-                for(i=n=0; i<filebox->getNumItems(); i++)
+                for(i=n=0; i<list->getNumItems(); i++)
                 {
-                    if(filebox->isItemSelected(i) && filebox->isItemFile(i))
-                        files[n++]=filebox->getItemPathname(i);
+                    if(list->isItemSelected(i) && list->isItemFile(i))
+                        files[n++]=list->getItemPathname(i);
                 }
                 files[n]=FXString::null;
             }
@@ -920,14 +982,14 @@ FXString FileSelector::getPatternList() const
 void FileSelector::setPattern(const FXString& ptrn)
 {
     filefilter->setText(ptrn);
-    filebox->setPattern(ptrn);
+    list->setPattern(ptrn);
 }
 
 
 // Get current filter pattern
 FXString FileSelector::getPattern() const
 {
-    return filebox->getPattern();
+    return list->getPattern();
 }
 
 
@@ -937,7 +999,7 @@ void FileSelector::setCurrentPattern(FXint patno)
     if(patno<0 || patno>=filefilter->getNumItems())
         fxerror("%s::setCurrentPattern: index out of range.\n",getClassName());
     filefilter->setCurrentItem(patno);
-    filebox->setPattern(patternFromText(filefilter->getItemText(patno)));
+    list->setPattern(patternFromText(filefilter->getItemText(patno)));
 }
 
 
@@ -971,28 +1033,28 @@ FXString FileSelector::getPatternText(FXint patno) const
 // Change space for item
 void FileSelector::setItemSpace(FXint s)
 {
-    filebox->setItemSpace(s);
+    list->setItemSpace(s);
 }
 
 
 // Get space for item
 FXint FileSelector::getItemSpace() const
 {
-    return filebox->getItemSpace();
+    return list->getItemSpace();
 }
 
 
 // Change File List style
 void FileSelector::setFileBoxStyle(FXuint style)
 {
-    filebox->setListStyle(style);
+    list->setListStyle(style);
 }
 
 
 // Return File List style
 FXuint FileSelector::getFileBoxStyle() const
 {
-    return filebox->getListStyle();
+    return list->getListStyle();
 }
 
 
@@ -1001,22 +1063,25 @@ void FileSelector::setSelectMode(FXuint mode)
 {
     switch(mode)
     {
-    case SELECTFILE_EXISTING:
-        filebox->showOnlyDirectories(FALSE);
-        filebox->setListStyle((filebox->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
+    case SELECT_FILE_EXISTING:
+        list->showOnlyDirectories(FALSE);
+        list->setListStyle((list->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
         break;
-    case SELECTFILE_MULTIPLE:
-    case SELECTFILE_MULTIPLE_ALL:
-        filebox->showOnlyDirectories(FALSE);
-        filebox->setListStyle((filebox->getListStyle()&~FILELISTMASK)|ICONLIST_EXTENDEDSELECT);
+    case SELECT_FILE_MULTIPLE:
+    case SELECT_FILE_MULTIPLE_ALL:
+        list->showOnlyDirectories(FALSE);
+        list->setListStyle((list->getListStyle()&~FILELISTMASK)|ICONLIST_EXTENDEDSELECT);
         break;
-    case SELECTFILE_DIRECTORY:
-        filebox->showOnlyDirectories(TRUE);
-        filebox->setListStyle((filebox->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
+    case SELECT_FILE_DIRECTORY:
+        list->showOnlyDirectories(TRUE);
+        list->setListStyle((list->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
+        break;
+    case SELECT_FILE_MIXED:
+        list->setListStyle((list->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
         break;
     default:
-        filebox->showOnlyDirectories(FALSE);
-        filebox->setListStyle((filebox->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
+        list->showOnlyDirectories(FALSE);
+        list->setListStyle((list->getListStyle()&~FILELISTMASK)|ICONLIST_BROWSESELECT);
         break;
     }
     selectmode=mode;
@@ -1064,7 +1129,7 @@ FileSelector::~FileSelector()
     	table->removeAccel(MKUINT(KEY_a,CONTROLMASK));
     	table->removeAccel(MKUINT(KEY_i,CONTROLMASK));
     }
-    delete filebox;
+    delete list;
 	delete filename;
 	delete filefilter;
 	delete readonly;
@@ -1076,172 +1141,213 @@ FileSelector::~FileSelector()
 }
 
 
+// File selector context menu
+long FileSelector::onCmdPopupMenu(FXObject* o,FXSelector s,void* p)
+{
+    // Popup menu pane
+	FXMenuPane menu(this);
+    FXint x,y;
+    FXuint state;
+    getRoot()->getCursorPosition(x,y,state);
+
+	new FXMenuCommand(&menu,_("Go ho&me"),homeicon,this,ID_HOME);
+	new FXMenuCommand(&menu,_("Go &work"),workicon,this,ID_WORK);
+	new FXMenuCommand(&menu,_("New f&older..."),newfoldericon,this,ID_NEWDIR);
+	new FXMenuSeparator(&menu);
+	new FXMenuCheck(&menu,_("&Hidden files"),list,FileList::ID_TOGGLE_HIDDEN);
+	new FXMenuCheck(&menu,_("Thum&bnails"),list,FileList::ID_TOGGLE_THUMBNAILS);
+	new FXMenuSeparator(&menu);
+	new FXMenuRadio(&menu,_("B&ig icons"),list,FXIconList::ID_SHOW_BIG_ICONS);
+	new FXMenuRadio(&menu,_("&Small icons"),list,FXIconList::ID_SHOW_MINI_ICONS);
+	new FXMenuRadio(&menu,_("F&ull file list"),list,FXIconList::ID_SHOW_DETAILS);
+	new FXMenuSeparator(&menu);
+	new FXMenuRadio(&menu,_("&Rows"),list,FileList::ID_ARRANGE_BY_ROWS);
+	new FXMenuRadio(&menu,_("&Columns"),list,FileList::ID_ARRANGE_BY_COLUMNS);
+	new FXMenuSeparator(&menu);
+	new FXMenuRadio(&menu,_("&Name"),list,FileList::ID_SORT_BY_NAME);
+	new FXMenuRadio(&menu,_("Si&ze"),list,FileList::ID_SORT_BY_SIZE);
+	new FXMenuRadio(&menu,_("&Type"),list,FileList::ID_SORT_BY_TYPE);
+	new FXMenuRadio(&menu,_("E&xtension"),list,FileList::ID_SORT_BY_EXT);
+	new FXMenuRadio(&menu,_("&Date"),list,FileList::ID_SORT_BY_TIME);
+	new FXMenuSeparator(&menu);
+	new FXMenuCheck(&menu,_("I&gnore case"),list,FileList::ID_SORT_CASE);
+	new FXMenuCheck(&menu,_("Dir&ectories first"),list,FileList::ID_DIRS_FIRST);
+	new FXMenuCheck(&menu,_("Re&verse order"),list,FileList::ID_SORT_REVERSE);
+
+	menu.create();
+	allowPopupScroll=TRUE;  // Allow keyboard scrolling
+	menu.popup(NULL,x,y);
+	getApp()->runModalWhileShown(&menu);
+	allowPopupScroll=FALSE;
+	return 1;
+}
+
 
 
 // Object implementation
-FXIMPLEMENT(FileDialog,FXDialogBox,NULL,0)
+FXIMPLEMENT(FileDialog,DialogBox,NULL,0)
 
 
 // File Dialog object
 FileDialog::FileDialog(FXWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h):
-        FXDialogBox(owner,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,w,h,0,0,0,0,4,4)
+        DialogBox(owner,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,w,h,0,0,0,0,4,4)
 {
-    filebox=new FileSelector(this,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    filebox->acceptButton()->setTarget(this);
-    filebox->acceptButton()->setSelector(FXDialogBox::ID_ACCEPT);
-    filebox->cancelButton()->setTarget(this);
-    filebox->cancelButton()->setSelector(FXDialogBox::ID_CANCEL);
+    list=new FileSelector(this,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	list->acceptButton()->setTarget(this);
+    list->acceptButton()->setSelector(DialogBox::ID_ACCEPT);
+    list->cancelButton()->setTarget(this);
+    list->cancelButton()->setSelector(DialogBox::ID_CANCEL);
 }
 
 
 // Set file name
 void FileDialog::setFilename(const FXString& path)
 {
-    filebox->setFilename(path);
+    list->setFilename(path);
 }
 
 
 // Get filename, if any
 FXString FileDialog::getFilename() const
 {
-    return filebox->getFilename();
+    return list->getFilename();
 }
 
 
 // Return empty-string terminated list of selected file names,
 FXString* FileDialog::getFilenames() const
 {
-    return filebox->getFilenames();
+    return list->getFilenames();
 }
 
 
 // Set pattern
 void FileDialog::setPattern(const FXString& ptrn)
 {
-    filebox->setPattern(ptrn);
+    list->setPattern(ptrn);
 }
 
 
 // Get pattern
 FXString FileDialog::getPattern() const
 {
-    return filebox->getPattern();
+    return list->getPattern();
 }
 
 
 // Change patterns, each pattern separated by newline
 void FileDialog::setPatternList(const FXString& patterns)
 {
-    filebox->setPatternList(patterns);
+    list->setPatternList(patterns);
 }
 
 
 // Return list of patterns
 FXString FileDialog::getPatternList() const
 {
-    return filebox->getPatternList();
+    return list->getPatternList();
 }
 
 
 // Set directory
 void FileDialog::setDirectory(const FXString& path)
 {
-    filebox->setDirectory(path);
+    list->setDirectory(path);
 }
 
 
 // Get directory
 FXString FileDialog::getDirectory() const
 {
-    return filebox->getDirectory();
+    return list->getDirectory();
 }
 
 
 // Set current file pattern from the list
 void FileDialog::setCurrentPattern(FXint n)
 {
-    filebox->setCurrentPattern(n);
+    list->setCurrentPattern(n);
 }
 
 
 // Return current pattern
 FXint FileDialog::getCurrentPattern() const
 {
-    return filebox->getCurrentPattern();
+    return list->getCurrentPattern();
 }
 
 FXString FileDialog::getPatternText(FXint patno) const
 {
-    return filebox->getPatternText(patno);
+    return list->getPatternText(patno);
 }
 
 
 void FileDialog::setPatternText(FXint patno,const FXString& text)
 {
-    filebox->setPatternText(patno,text);
+    list->setPatternText(patno,text);
 }
 
 
 // Set list of patterns (DEPRECATED)
 void FileDialog::setPatternList(const FXchar **ptrns)
 {
-    filebox->setPatternList(ptrns);
+    list->setPatternList(ptrns);
 }
 
 
 // Change space for item
 void FileDialog::setItemSpace(FXint s)
 {
-    filebox->setItemSpace(s);
+    list->setItemSpace(s);
 }
 
 
 // Get space for item
 FXint FileDialog::getItemSpace() const
 {
-    return filebox->getItemSpace();
+    return list->getItemSpace();
 }
 
 
 // Change File List style
 void FileDialog::setFileBoxStyle(FXuint style)
 {
-    filebox->setFileBoxStyle(style);
+    list->setFileBoxStyle(style);
 }
 
 
 // Return File List style
 FXuint FileDialog::getFileBoxStyle() const
 {
-    return filebox->getFileBoxStyle();
+    return list->getFileBoxStyle();
 }
 
 
 // Change file selection mode
 void FileDialog::setSelectMode(FXuint mode)
 {
-    filebox->setSelectMode(mode);
+    list->setSelectMode(mode);
 }
 
 
 // Return file selection mode
 FXuint FileDialog::getSelectMode() const
 {
-    return filebox->getSelectMode();
+    return list->getSelectMode();
 }
 
 
 // Show readonly button
 void FileDialog::showReadOnly(FXbool show)
 {
-    filebox->showReadOnly(show);
+    list->showReadOnly(show);
 }
 
 
 // Return TRUE if readonly is shown
 FXbool FileDialog::shownReadOnly() const
 {
-    return filebox->shownReadOnly();
+    return list->shownReadOnly();
 }
 
 
@@ -1249,21 +1355,21 @@ FXbool FileDialog::shownReadOnly() const
 // Set initial state of readonly button
 void FileDialog::setReadOnly(FXbool state)
 {
-    filebox->setReadOnly(state);
+    list->setReadOnly(state);
 }
 
 
 // Get readonly state
 FXbool FileDialog::getReadOnly() const
 {
-    return filebox->getReadOnly();
+    return list->getReadOnly();
 }
 
 
 // Cleanup
 FileDialog::~FileDialog()
 {
-    delete filebox;
+    delete list;
 }
 
 
@@ -1272,7 +1378,7 @@ FXString FileDialog::getOpenFilename(FXWindow* owner,const FXString& caption,con
 {
     FileDialog opendialog(owner,caption);
     FXString filename;
-    opendialog.setSelectMode(SELECTFILE_EXISTING);
+    opendialog.setSelectMode(SELECT_FILE_EXISTING);
     opendialog.setPatternList(patterns);
     opendialog.setCurrentPattern(initial);
     opendialog.setFilename(path);
@@ -1290,7 +1396,7 @@ FXString FileDialog::getOpenFilename(FXWindow* owner,const FXString& caption,con
 FXString FileDialog::getSaveFilename(FXWindow* owner,const FXString& caption,const FXString& path,const FXString& patterns,FXint initial)
 {
     FileDialog savedialog(owner,caption);
-    savedialog.setSelectMode(SELECTFILE_ANY);
+    savedialog.setSelectMode(SELECT_FILE_ANY);
     savedialog.setPatternList(patterns);
     savedialog.setCurrentPattern(initial);
     savedialog.setFilename(path);
@@ -1304,7 +1410,7 @@ FXString FileDialog::getSaveFilename(FXWindow* owner,const FXString& caption,con
 FXString* FileDialog::getOpenFilenames(FXWindow* owner,const FXString& caption,const FXString& path,const FXString& patterns,FXint initial)
 {
     FileDialog opendialog(owner,caption);
-    opendialog.setSelectMode(SELECTFILE_MULTIPLE);
+    opendialog.setSelectMode(SELECT_FILE_MULTIPLE);
     opendialog.setPatternList(patterns);
     opendialog.setCurrentPattern(initial);
     opendialog.setFilename(path);
@@ -1319,7 +1425,7 @@ FXString FileDialog::getOpenDirectory(FXWindow* owner,const FXString& caption,co
 {
     FileDialog dirdialog(owner,caption);
     FXString dirname;
-    dirdialog.setSelectMode(SELECTFILE_DIRECTORY);
+    dirdialog.setSelectMode(SELECT_FILE_DIRECTORY);
     dirdialog.setFilename(path);
     if(dirdialog.execute())
     {
