@@ -144,7 +144,7 @@ private:
     FileList &operator=(const FileList&);
 public:
 	long onCmdRefresh(FXObject*,FXSelector,void*);
-    long onRefreshTimer(FXObject*,FXSelector,void*);
+    long onCmdRefreshTimer(FXObject*,FXSelector,void*);
     long onOpenTimer(FXObject*,FXSelector,void*);
     long onDNDEnter(FXObject*,FXSelector,void*);
     long onDNDLeave(FXObject*,FXSelector,void*);
@@ -192,6 +192,10 @@ public:
 	long onUpdToggleThumbnails(FXObject* sender,FXSelector,void*);
 	long onCmdDirsFirst(FXObject*,FXSelector,void*);
 	long onUpdDirsFirst(FXObject*,FXSelector,void*);
+	long onCmdDragCopy(FXObject* sender,FXSelector,void*);
+	long onCmdDragMove(FXObject* sender,FXSelector,void*);
+	long onCmdDragLink(FXObject* sender,FXSelector,void*);
+	long onCmdDragReject(FXObject* sender,FXSelector,void*);
 	
 public:
 	static FXint ascending(const FXIconItem* pa,const FXIconItem* pb);
@@ -260,6 +264,10 @@ public:
         ID_REFRESH_TIMER,
 		ID_REFRESH,
         ID_OPEN_TIMER,
+		ID_DRAG_COPY,
+		ID_DRAG_MOVE,
+		ID_DRAG_LINK,
+		ID_DRAG_REJECT,
         ID_LAST
     };
 public:
@@ -280,9 +288,9 @@ public:
     FXString getCurrentFile() const;
 
     // Set current directory
-    void setDirectory(const FXString& path, const FXbool histupdate=TRUE);
+	void setDirectory(const FXString& path,const FXbool histupdate=TRUE,FXString prevpath= "");
 
-    // Return current directory
+	// Return current directory
     FXString getDirectory() const
     {
         return directory;
