@@ -3,12 +3,16 @@
 #include "Properties.h"
 #include "FileList.h"
 #include "PathLinker.h"
+#include "InputDialog.h"
+#include "BrowseInputDialog.h"
+#include "HistInputDialog.h"
+#include "ArchInputDialog.h"
 
 // Clipboard operations
 enum
 {	
-	COPY_CLP,
-	CUT_CLP,
+	COPY_CLIPBOARD,
+	CUT_CLIPBOARD,
 };
 
 class FilePanel : public FXVerticalFrame
@@ -32,10 +36,24 @@ protected:
 	FXString		homelocation;
 	FXString		trashlocation;
 	FXDragCorner    *corner;
-
+	FXDragType      urilistType;        // Standard uri-list type
+	FXDragType      xfelistType;        // Xfe, Gnome and XFCE list type
+	FXDragType      kdelistType;        // KDE list type
+	FXDragType      utf8Type;           // UTF-8 text type
+	FXbool 			clipboard_locked;	// Clipboard locked to prevent changes when viewing it
+	InputDialog*    newfiledialog;
+	InputDialog*    newdirdialog;
+	HistInputDialog* opendialog;
+	ArchInputDialog* archdialog;
+	HistInputDialog* filterdialog;
+	BrowseInputDialog* operationdialog;
+	FXbool			 paste;
 
 public:
-    FilePanel(FXchar*,FXComposite*, FXuint name_size=200, FXuint size_size=60, FXuint type_size=100, FXuint ext_size=100, 
+
+
+
+FilePanel(FXchar*,FXComposite*, FXuint name_size=200, FXuint size_size=60, FXuint type_size=100, FXuint ext_size=100, 
 	      FXuint modd_size=150, FXuint user_size=50, FXuint grou_size=50, FXuint attr_size=100, FXColor listbackcolor=FXRGB(255,255,255), FXColor listforecolor=FXRGB(0,0,0),
  		  FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
     FilePanel()
@@ -54,6 +72,7 @@ public:
         ID_EDIT,
         ID_PROPERTIES,
         ID_FILE_COPY,
+		ID_FILE_CUT,
         ID_FILE_COPYTO,
         ID_FILE_MOVE,
         ID_FILE_RENAME,
@@ -64,15 +83,16 @@ public:
 		ID_MENU,
 		ID_XTERM,
         ID_EXTRACT,
+		ID_EXTRACT_TO_FOLDER,
         ID_EXTRACT_HERE,
         ID_NEW_DIR,
         ID_NEW_FILE,
         ID_ADD_TO_ARCH,
         ID_GO_HOME,
 		ID_GO_TRASH,
-        ID_COPY_CLP,
-        ID_CUT_CLP,
-        ID_PASTE_CLP,
+        ID_COPY_CLIPBOARD,
+        ID_CUT_CLIPBOARD,
+        ID_PASTE_CLIPBOARD,
         ID_OPEN,
         ID_OPEN_WITH,
         ID_FILTER,
@@ -97,12 +117,15 @@ public:
         ID_LAST,
     };
 public:
+	long onClipboardGained(FXObject*,FXSelector,void*);
+	long onClipboardLost(FXObject*,FXSelector,void*);
+	long onClipboardRequest(FXObject*,FXSelector,void*);
     long onUpdStatus(FXObject*,FXSelector,void*);
     long onCmdItemDoubleClicked(FXObject*,FXSelector,void*);
     long onCmdItemClicked(FXObject*,FXSelector,void*);
 	long onCmdItemFocus(FXObject*,FXSelector, void*);
     long onCmdItemFilter(FXObject*,FXSelector,void*);
-    long onCmdClp(FXObject*,FXSelector,void*);
+    long onCmdCopyCut(FXObject*,FXSelector,void*);
     long onCmdPaste(FXObject*,FXSelector,void*);
 	long onCmdDirectoryUp(FXObject*,FXSelector,void*);
     long onCmdGoHome(FXObject*,FXSelector,void*);
@@ -119,8 +142,9 @@ public:
     long onCmdOpenWith(FXObject*,FXSelector,void*);
 	long onCmdXTerm(FXObject*,FXSelector,void*);
     long onCmdExtract(FXObject*,FXSelector,void*);
+    long onCmdExtractToFolder(FXObject*,FXSelector,void*);
     long onCmdExtractHere(FXObject*,FXSelector,void*);
-    long onRefresh(FXObject*,FXSelector,void*);
+    long onCmdRefresh(FXObject*,FXSelector,void*);
     long onCmdSelect(FXObject*,FXSelector,void*);
     long onCmdPopupMenu(FXObject*,FXSelector,void*);
     long onCmdShow(FXObject*,FXSelector,void*);
@@ -138,7 +162,7 @@ public:
 	long onUpdSelMult(FXObject*,FXSelector,void*);
 	long onUpdFileDelete(FXObject*,FXSelector,void*);
 	long onUpdFileTrash(FXObject*,FXSelector,void*);
-	long onUpdGoTrash(FXObject* o,FXSelector sel,void*);
+	long onUpdGoTrash(FXObject*,FXSelector,void*);
 	void updatePathLinker();
 
 #if defined(linux)
