@@ -33,12 +33,12 @@ FXDEFMAP(MessageBox) MessageBoxMap[]={
 
 
 // Object implementation
-FXIMPLEMENT(MessageBox,FXDialogBox,MessageBoxMap,ARRAYNUMBER(MessageBoxMap))
+FXIMPLEMENT(MessageBox,DialogBox,MessageBoxMap,ARRAYNUMBER(MessageBoxMap))
 
 
 // Construct message box with given caption, icon, and message text
 MessageBox::MessageBox(FXWindow* owner,const FXString& caption,const FXString& text,FXIcon* ic,FXuint opts,FXint x,FXint y):
-        FXDialogBox(owner,caption,opts|DECOR_TITLE|DECOR_BORDER,x,y,0,0, 0,0,0,0, 4,4)
+        DialogBox(owner,caption,opts|DECOR_TITLE|DECOR_BORDER,x,y,0,0, 0,0,0,0, 4,4)
 {
     initialize(text,ic,opts&BOX_BUTTON_MASK);
 }
@@ -46,7 +46,7 @@ MessageBox::MessageBox(FXWindow* owner,const FXString& caption,const FXString& t
 
 // Construct free floating message box with given caption, icon, and message text
 MessageBox::MessageBox(FXApp* a,const FXString& caption,const FXString& text,FXIcon* ic,FXuint opts,FXint x,FXint y):
-        FXDialogBox(a,caption,opts|DECOR_TITLE|DECOR_BORDER,x,y,0,0, 0,0,0,0, 4,4)
+        DialogBox(a,caption,opts|DECOR_TITLE|DECOR_BORDER,x,y,0,0, 0,0,0,0, 4,4)
 {
     initialize(text,ic,opts&BOX_BUTTON_MASK);
 }
