@@ -22,12 +22,12 @@ FXDEFMAP(ConfirmOverWriteBox) ConfirmOverWriteBoxMap[]={
 
 
 // Object implementation
-FXIMPLEMENT(ConfirmOverWriteBox,FXDialogBox,ConfirmOverWriteBoxMap,ARRAYNUMBER(ConfirmOverWriteBoxMap))
+FXIMPLEMENT(ConfirmOverWriteBox,DialogBox,ConfirmOverWriteBoxMap,ARRAYNUMBER(ConfirmOverWriteBoxMap))
 
 
 // Create message box
 ConfirmOverWriteBox::ConfirmOverWriteBox(FXWindow *win,const FXString& name,const FXString& text,FXuint opts,FXint x,FXint y):
-        FXDialogBox(win,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,0,0)
+        DialogBox(win,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,0,0)
 {
     FXVerticalFrame* content=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
     FXHorizontalFrame* info=new FXHorizontalFrame(content,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10);
