@@ -6,8 +6,10 @@
 #include <fox-1.6/fx.h>
 #include <fox-1.6/fxkeys.h>
 
+#include "icons.h"
 #include "xfedefs.h"
 #include "FileDialog.h"
+#include "FontDialog.h"
 #include "Preferences.h"
 #include "XFileExplorer.h"
 #include "MessageBox.h"
@@ -41,14 +43,21 @@ FXDEFMAP(PreferencesBox) PreferencesMap[]={
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_TEXTFONT,PreferencesBox::onCmdTextFont),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_THEME,PreferencesBox::onCmdTheme),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_BROWSE_ICON_PATH,PreferencesBox::onCmdBrowsePath),
-                                         FXMAPFUNC(SEL_UPDATE ,PreferencesBox::ID_COLOR,PreferencesBox::onUpdColor),
+                                         FXMAPFUNC(SEL_UPDATE,PreferencesBox::ID_COLOR,PreferencesBox::onUpdColor),
 									     FXMAPFUNC(SEL_UPDATE,PreferencesBox::ID_TRASH_BYPASS,PreferencesBox::onUpdTrashBypass),
+                                         FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_SU_CMD,PreferencesBox::onCmdSuMode),
+                                         FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_SUDO_CMD,PreferencesBox::onCmdSuMode),
+                                         FXMAPFUNC(SEL_UPDATE,PreferencesBox::ID_SU_CMD,PreferencesBox::onUpdSuMode),
+                                         FXMAPFUNC(SEL_UPDATE,PreferencesBox::ID_SUDO_CMD,PreferencesBox::onUpdSuMode),
+                                         FXMAPFUNC(SEL_UPDATE,PreferencesBox::ID_COLOR,PreferencesBox::onUpdColor),
+                                         FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_WHEELADJUST,PreferencesBox::onCmdWheelAdjust),
+                                         FXMAPFUNC(SEL_UPDATE,PreferencesBox::ID_WHEELADJUST,PreferencesBox::onUpdWheelAdjust),
                                      };
 // Object implementation
-FXIMPLEMENT(PreferencesBox,FXDialogBox,PreferencesMap,ARRAYNUMBER(PreferencesMap))
+FXIMPLEMENT(PreferencesBox,DialogBox,PreferencesMap,ARRAYNUMBER(PreferencesMap))
 
 // Construct window for one file
-PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor listforecolor, FXColor highlightcolor): FXDialogBox(win,_("Preferences"), DECOR_TITLE|DECOR_BORDER)
+PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor listforecolor, FXColor highlightcolor): DialogBox(win,_("Preferences"), DECOR_TITLE|DECOR_BORDER)
 {
     currTheme.name = _("Current Theme");
     currTheme.color[0] = getApp()->getBaseColor();
@@ -93,7 +102,7 @@ PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor lis
 
 
     // First tab - Options
-    new FXTabItem(tabbook,_("&Options"),NULL);
+    new FXTabItem(tabbook,_("&General"),NULL);
     FXVerticalFrame *options=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
 
 	FXGroupBox *group=new FXGroupBox(options,_("Options"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
@@ -103,24 +112,21 @@ PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor lis
     diropen=new FXCheckButton(group,_("Single click directory open"));
     fileopen=new FXCheckButton(group,_("Single click file open"));
 
-	group=new FXGroupBox(options,_("Confirmations"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    ask=new FXCheckButton(group,_("Always ask before file operation"));
-	del=new FXCheckButton(group,_("Confirm delete"));
-    overwrite=new FXCheckButton(group,_("Confirm overwrite"));
-	quit=new FXCheckButton(group,_("Confirm quit"));
+	group=new FXGroupBox(options,_("Mouse"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    FXMatrix *matrix = new FXMatrix(group,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	new FXLabel(matrix,_("Mouse scrolling speed:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    FXSpinner* spinner=new FXSpinner(matrix,3,this,PreferencesBox::ID_WHEELADJUST,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_ROW,0,0,0,0, 2,2,1,1);
+    spinner->setRange(1,100);
 
-	group=new FXGroupBox(options,_("Warnings"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-#if defined(linux)
-    mount=new FXCheckButton (group,_("Warn when mount points are not responding"));
-	show_mount=new FXCheckButton(group, _("Display mount/umount success messages"));
-#endif
-    root_warning=new FXCheckButton (group,_("Warn if running as root"));
-
-    FXbool root_warn=getApp()->reg().readUnsignedEntry("OPTIONS","root_warn",TRUE);
-	if(getuid())//running as a user?
-        root_warning->disable();
-    else
-        root_warning->setCheck(root_warn);
+	group=new FXGroupBox(options,_("Root mode"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	FXRadioButton* subutton=new FXRadioButton(group,_("Authentication using su (uses root password)"),this,ID_SU_CMD);
+	FXRadioButton* sudobutton=new FXRadioButton(group,_("Authentication using sudo (uses user password)"),this,ID_SUDO_CMD);
+	if (getuid()==0) // Super user
+	{
+		subutton->disable();
+		sudobutton->disable();
+	}
+	use_sudo=getApp()->reg().readUnsignedEntry("OPTIONS","use_sudo",FALSE);
 
     FXbool use_trash_can=getApp()->reg().readUnsignedEntry("OPTIONS","use_trash_can",TRUE);
 	trashcan->setCheck(use_trash_can);
@@ -135,6 +141,27 @@ PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor lis
 
 	FXbool auto_save_layout=getApp()->reg().readUnsignedEntry("OPTIONS","auto_save_layout",TRUE);
 	autosave->setCheck(auto_save_layout);
+    single_click_diropen=getApp()->reg().readUnsignedEntry("OPTIONS","single_click_diropen",FALSE);
+	diropen->setCheck(single_click_diropen);
+    single_click_fileopen=getApp()->reg().readUnsignedEntry("OPTIONS","single_click_fileopen",FALSE);
+	fileopen->setCheck(single_click_fileopen);
+
+    // Second tab - Dialogs
+    new FXTabItem(tabbook,_("&Dialogs"),NULL);
+    FXVerticalFrame *dialogs=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
+	group=new FXGroupBox(dialogs,_("Confirmations"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    ask=new FXCheckButton(group,_("Always ask before file operation"));
+	del=new FXCheckButton(group,_("Confirm delete"));
+    overwrite=new FXCheckButton(group,_("Confirm overwrite"));
+	quit=new FXCheckButton(group,_("Confirm quit"));
+
+	group=new FXGroupBox(dialogs,_("Warnings"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+#if defined(linux)
+    mount=new FXCheckButton (group,_("Warn when mount points are not responding"));
+	show_mount=new FXCheckButton(group, _("Display mount/umount success messages"));
+#endif
+    root_warning=new FXCheckButton (group,_("Warn if running as root"));
+
 	FXbool confirm_quit=getApp()->reg().readUnsignedEntry("OPTIONS","confirm_quit",TRUE);
 	quit->setCheck(confirm_quit);
     FXbool confirm_del=getApp()->reg().readUnsignedEntry("OPTIONS","confirm_delete",TRUE);
@@ -143,10 +170,7 @@ PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor lis
 	overwrite->setCheck(confirm_overwrite);
     FXbool ask_before_copy=getApp()->reg().readUnsignedEntry("OPTIONS","ask_before_copy",TRUE);
 	ask->setCheck(ask_before_copy);
-    single_click_diropen=getApp()->reg().readUnsignedEntry("OPTIONS","single_click_diropen",FALSE);
-	diropen->setCheck(single_click_diropen);
-    single_click_fileopen=getApp()->reg().readUnsignedEntry("OPTIONS","single_click_fileopen",FALSE);
-	fileopen->setCheck(single_click_fileopen);
+
 #if defined(linux)
 	FXbool mount_warn=getApp()->reg().readUnsignedEntry("OPTIONS","mount_warn",TRUE);
 	FXbool mount_messages=getApp()->reg().readUnsignedEntry("OPTIONS","mount_messages",TRUE);
@@ -154,37 +178,55 @@ PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor lis
 	show_mount->setCheck(mount_messages);
 #endif
 
-    // Second tab - Programs
+	FXbool root_warn=getApp()->reg().readUnsignedEntry("OPTIONS","root_warn",TRUE);
+	if(getuid()) // Simple user
+        root_warning->disable();
+    else
+        root_warning->setCheck(root_warn);
+
+	// Third tab - Programs
     new FXTabItem(tabbook,_("&Programs"),NULL);
     FXVerticalFrame *programs=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
     group=new FXGroupBox(programs,_("Programs"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    FXMatrix *matrix = new FXMatrix(group,3,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    matrix = new FXMatrix(group,3,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
 
     new FXLabel(matrix,_("Default text editor:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    txteditor = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_TXTEDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);//,0,0,0,0,20,20);
+    txteditor = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_TXTEDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
     oldtxteditor=getApp()->reg().readStringEntry("PROGS","txteditor","xfw");
     txteditor->setText(oldtxteditor);
 
 	new FXLabel(matrix,_("Default text viewer:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    txtviewer = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-	new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_TXTVIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);//0,0,0,0,20,20);
+    txtviewer = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+	new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_TXTVIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
     oldtxtviewer=getApp()->reg().readStringEntry("PROGS","txtviewer","xfv");
     txtviewer->setText(oldtxtviewer);
 
+	new FXLabel(matrix,_("Default image editor:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    imgeditor = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+	new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_IMGVIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    oldimgeditor=getApp()->reg().readStringEntry("PROGS","imgeditor","gimp");
+    imgeditor->setText(oldimgeditor);
+
 	new FXLabel(matrix,_("Default image viewer:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    imgviewer = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-	new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_IMGVIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);//0,0,0,0,20,20);
+    imgviewer = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+	new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_IMGVIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
     oldimgviewer=getApp()->reg().readStringEntry("PROGS","imgviewer","xfi");
     imgviewer->setText(oldimgviewer);
 
-    new FXLabel(matrix,_("Terminal program:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    xterm = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_XTERM,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);//,0,0,0,0,20,20);
+	new FXLabel(matrix,_("Default archiver:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    archiver = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+	new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_IMGVIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    oldarchiver=getApp()->reg().readStringEntry("PROGS","archiver","file-roller");
+    archiver->setText(oldarchiver);
+
+	new FXLabel(matrix,_("Terminal program:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    xterm = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_XTERM,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
     oldxterm=getApp()->reg().readStringEntry("PROGS","xterm","xterm -sb");
     xterm->setText(oldxterm);
 
-	// Third tab - Visual
+	// Fourth tab - Visual
     new FXTabItem(tabbook,_("&Themes"),NULL);
     FXVerticalFrame *visual=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
 
@@ -217,25 +259,25 @@ PreferencesBox::PreferencesBox(FXWindow *win, FXColor listbackcolor, FXColor lis
 	// Find iconpath from the Xfe registry settings or set it to DEFAULTICONPATH
 	FXGroupBox *group2=new FXGroupBox(visual,_("Icon theme path"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
 	FXMatrix *matrix2 = new FXMatrix(group2,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-	iconpath = new FXTextField(matrix2,40,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN);
-    new FXButton(matrix2,_(" Select..."),NULL,this,ID_BROWSE_ICON_PATH,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
+	iconpath = new FXTextField(matrix2,40,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix2,_("\tSelect path..."),filedialogicon,this,ID_BROWSE_ICON_PATH,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 	oldiconpath=getApp()->reg().readStringEntry("SETTINGS","iconpath",DEFAULTICONPATH);
     iconpath->setText(oldiconpath);
 
-    // Fourth tab - Fonts
+    // Fifth tab - Fonts
     new FXTabItem(tabbook,_("&Fonts"),NULL);
     FXVerticalFrame *fonts=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
     FXGroupBox *fgroup=new FXGroupBox(fonts,_("Fonts"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
 
     FXMatrix *fmatrix = new FXMatrix(fgroup,3,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
     new FXLabel(fmatrix,_("Normal font:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    normalfont = new FXTextField(fmatrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    normalfont = new FXTextField(fmatrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
     new FXButton(fmatrix,_(" Select..."),NULL,this,ID_NORMALFONT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,20,20);
     oldnormalfont=getApp()->reg().readStringEntry("SETTINGS","font","Helvetica,100,normal,regular");
     normalfont->setText(oldnormalfont);
 
     new FXLabel(fmatrix,_("Text font:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    textfont = new FXTextField(fmatrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    textfont = new FXTextField(fmatrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
 	new FXButton(fmatrix,_(" Select..."),NULL,this,ID_TEXTFONT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//0,0,0,0,20,20);
     oldtextfont=getApp()->reg().readStringEntry("SETTINGS","textfont","Helvetica,100,normal,regular");
     textfont->setText(oldtextfont);
@@ -274,27 +316,23 @@ long PreferencesBox::onCmdTheme(FXObject* o,FXSelector s,void* p)
 
 long PreferencesBox::onCmdBrowsePath(FXObject* o,FXSelector s,void* p)
 {
-   FileDialog browse(this,_("Select an icon file to set the icon path"),0,0,0,600,400);
+	FileDialog browse(this,_("Select an icon theme directory"),0,0,0,650,480);
 
-    const FXchar *patterns[]=
-        {
-            _("PNG Images"),     "*.png",NULL
-        };
-   browse.setFilename(iconpath->getText());
-   browse.setPatternList(patterns);
+	browse.setSelectMode(SELECT_FILE_DIRECTORY);
+	browse.setFilename(iconpath->getText());
 
-   if(browse.execute())
-   {
-      FXString path=browse.getFilename();
-      iconpath->setText(FXPath::directory(path).text());
-   }
-   return 1;
+	if(browse.execute())
+	{
+		FXString path=browse.getFilename();
+		iconpath->setText(path);
+	}
+	return 1;
 }
 
 
 long PreferencesBox::onCmdBrowse(FXObject* o,FXSelector s,void* p)
 {
-    FileDialog browse(this,_("Select Program"),0,0,0,600,400);
+    FileDialog browse(this,_("Select Program"),0,0,0,650,480);
     const FXchar *patterns[]=
         {
             _("All files"),     "*",NULL
@@ -326,12 +364,13 @@ long PreferencesBox::onCmdBrowse(FXObject* o,FXSelector s,void* p)
 // Change normal font
 long PreferencesBox::onCmdNormalFont(FXObject*,FXSelector,void*)
 {
-    FXFontDialog fontdlg(this,_("Change Normal Font"),DECOR_BORDER|DECOR_TITLE);
+    FontDialog fontdlg(this,_("Change Normal Font"),DECOR_BORDER|DECOR_TITLE);
     FXFontDesc fontdesc;
 	FXString fontspec;
 
 	fontspec=getApp()->reg().readStringEntry("SETTINGS","font","Helvetica,100,normal,regular");
    	FXFont* nfont=new FXFont(getApp(),fontspec);
+	nfont->create();
 	nfont->getFontDesc(fontdesc);
     
 	fontdlg.setFontSelection(fontdesc);
@@ -354,11 +393,12 @@ long PreferencesBox::onCmdNormalFont(FXObject*,FXSelector,void*)
 // Change text font
 long PreferencesBox::onCmdTextFont(FXObject*,FXSelector,void*)
 {
-    FXFontDialog fontdlg(this,_("Change Text Font"),DECOR_BORDER|DECOR_TITLE);
+    FontDialog fontdlg(this,_("Change Text Font"),DECOR_BORDER|DECOR_TITLE);
     FXFontDesc fontdesc;
 	FXString fontspec;
 	fontspec=getApp()->reg().readStringEntry("SETTINGS","textfont","Helvetica,100,normal,regular");
    	FXFont* tfont=new FXFont(getApp(),fontspec);
+	tfont->create();
 	tfont->getFontDesc(fontdesc);
     fontdlg.setFontSelection(fontdesc);
     if(fontdlg.execute())
@@ -378,14 +418,16 @@ long PreferencesBox::onCmdTextFont(FXObject*,FXSelector,void*)
 
 long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
 {
-    if(oldiconpath != iconpath->getText())
+    FXbool restart=FALSE;
+	
+	if(oldiconpath != iconpath->getText())
 	{
 		getApp()->reg().writeStringEntry("SETTINGS","iconpath",iconpath->getText().text());
-        if(BOX_CLICKED_CANCEL!=MessageBox::question(this,BOX_OK_CANCEL,_("Restart"),_("Icon theme will be changed after restart.\nRestart X File Explorer now?")))
-            mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_RESTART),NULL);
+ 		getApp()->reg().write();
+		restart=TRUE;
 	}
 
-    // Editor has changed
+    // Text editor has changed
 	if(oldtxteditor != txteditor->getText())
 	{
         // Update the txteditor string
@@ -426,9 +468,6 @@ long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
 				assoc->replace(key.text(),value.text());
 			}
 		}
-		
-		// Update the registry
-		//getApp()->reg().write();
 	}
 
     // Text viewer has changed
@@ -470,9 +509,49 @@ long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
 				assoc->replace(key.text(),value.text());
 			}
 		}
+	}
+
+    // Image editor has changed
+	if(oldimgeditor != imgeditor->getText())
+	{
+        // Update the imgeditor string
+		FXString newimgeditor=imgeditor->getText().text();
+		getApp()->reg().writeStringEntry("PROGS","imgeditor",newimgeditor.text());
 		
-		// Update the registry
-		//getApp()->reg().write();
+		// Update each filetype where the old imgeditor was used
+		FXStringDict *strdict = getApp()->reg().find("FILETYPES");
+		FileDict *assoc=new FileDict(getApp());
+
+		FXint i;
+		FXString key, value, newvalue;
+		FXString strtmp, open, view, edit, command;
+		for (i=strdict->first(); i<strdict->size(); i=strdict->next(i))
+		{
+			// Read key and value of each filetype
+			key=strdict->key(i);
+			value=strdict->data(i);
+			
+			// Replace the old imgeditor string with the new one
+			if (value.contains(oldimgeditor))
+			{
+				// Obtain the open, view, edit and command strings
+				strtmp=value.before(';',1);
+				command=value.after(';',1);
+				open=strtmp.section(',',0);
+				view=strtmp.section(',',1);
+				edit=strtmp.section(',',2);
+				
+				// Replace only the open and edit strings, if needed
+				if (open==oldimgeditor)
+					open=newimgeditor;
+				if (edit==oldimgeditor)
+					edit=newimgeditor;
+				
+				// Replace with the new value
+				value=open + "," + view + "," + edit + ";" + command;
+				assoc->replace(key.text(),value.text());
+			}
+		}
 	}
 
     // Image viewer has changed
@@ -514,9 +593,51 @@ long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
 				assoc->replace(key.text(),value.text());
 			}
 		}
+	}
+
+    // Archiver has changed
+	if(oldarchiver != archiver->getText())
+	{
+        // Update the archiver string
+		FXString newarchiver=archiver->getText().text();
+		getApp()->reg().writeStringEntry("PROGS","archiver",newarchiver.text());
 		
-		// Update the registry
-		//getApp()->reg().write();
+		// Update each filetype where the old archiver was used
+		FXStringDict *strdict = getApp()->reg().find("FILETYPES");
+		FileDict *assoc=new FileDict(getApp());
+
+		FXint i;
+		FXString key, value, newvalue;
+		FXString strtmp, open, view, edit, command;
+		for (i=strdict->first(); i<strdict->size(); i=strdict->next(i))
+		{
+			// Read key and value of each filetype
+			key=strdict->key(i);
+			value=strdict->data(i);
+			
+			// Replace the old archiver string with the new one
+			if (value.contains(oldarchiver))
+			{
+				// Obtain the open, view, edit and command strings
+				strtmp=value.before(';',1);
+				command=value.after(';',1);
+				open=strtmp.section(',',0);
+				view=strtmp.section(',',1);
+				edit=strtmp.section(',',2);
+				
+				// Replace the open, view and edit strings, if needed
+				if (open==oldarchiver)
+					open=newarchiver;
+				if (view==oldarchiver)
+					view=newarchiver;
+				if (edit==oldarchiver)
+					edit=newarchiver;
+				
+				// Replace with the new value
+				value=open + "," + view + "," + edit + ";" + command;
+				assoc->replace(key.text(),value.text());
+			}
+		}
 	}
 
 	// Terminal has changed
@@ -568,14 +689,21 @@ long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
 		getApp()->reg().writeColorEntry("SETTINGS","listforecolor",currTheme.color[7]);
 		getApp()->reg().writeColorEntry("SETTINGS","highlightcolor",currTheme.color[8]);
 
-        if(BOX_CLICKED_CANCEL!=MessageBox::question(this,BOX_OK_CANCEL,_("Restart"),_("Colors will be changed after restart.\nRestart X File Explorer now?")))
-            mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_RESTART),NULL);
+		getApp()->reg().write();
+        restart=TRUE;
     }
 	
-	// Update the registry
+	// Restart application if necessary
+	if (restart)
+	{
+		if(BOX_CLICKED_CANCEL!=MessageBox::question(this,BOX_OK_CANCEL,_("Restart"),_("Theme will be changed after restart.\nRestart X File Explorer now?")))
+			mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_RESTART),NULL);
+	}
+	
+	// Finally, update the registry
 	getApp()->reg().write();
 	
-    FXDialogBox::onCmdAccept(o,s,p);
+    DialogBox::onCmdAccept(o,s,p);
     return 1;
 }
 
@@ -589,3 +717,63 @@ long PreferencesBox::onUpdTrashBypass(FXObject* o,FXSelector,void*)
 		o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
     return 1;
 }
+
+
+// Set root mode
+long PreferencesBox::onCmdSuMode(FXObject*,FXSelector sel,void*)
+{
+	if (FXSELID(sel)==ID_SU_CMD)
+		use_sudo=FALSE;
+		
+	else if (FXSELID(sel)==ID_SUDO_CMD)
+		use_sudo=TRUE;
+
+	getApp()->reg().writeUnsignedEntry("OPTIONS","use_sudo",use_sudo);
+	getApp()->reg().write();
+
+	return 1;
+}
+
+
+// Update root mode radio button
+long PreferencesBox::onUpdSuMode(FXObject* sender,FXSelector sel,void*)
+{
+	FXSelector updatemessage=FXSEL(SEL_COMMAND,ID_UNCHECK);
+	
+	if (FXSELID(sel)==ID_SU_CMD)
+	{
+		if (use_sudo)
+			updatemessage=FXSEL(SEL_COMMAND,ID_UNCHECK);
+		else
+			updatemessage=FXSEL(SEL_COMMAND,ID_CHECK);
+	}
+	else if (FXSELID(sel)==ID_SUDO_CMD)
+	{
+		if (use_sudo)
+			updatemessage=FXSEL(SEL_COMMAND,ID_CHECK);
+		else
+			updatemessage=FXSEL(SEL_COMMAND,ID_UNCHECK);
+
+	}		
+	sender->handle(this,updatemessage,NULL);
+	return 1;
+}
+
+// Set scroll wheel lines (Mathew Robertson <mathew@optushome.com.au>)
+long PreferencesBox::onCmdWheelAdjust(FXObject* sender,FXSelector,void*)
+{
+    FXuint value;
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_GETINTVALUE),(void*)&value);
+    getApp()->setWheelLines(value);
+	getApp()->reg().write();
+    return 1;
+}
+
+
+// Update wheel lines
+long PreferencesBox::onUpdWheelAdjust(FXObject* sender,FXSelector,void*)
+{
+    FXuint value=getApp()->getWheelLines();
+	sender->handle(this,FXSEL(SEL_COMMAND,ID_SETINTVALUE),(void*)&value);
+    return 1;
+}
