@@ -3,6 +3,8 @@
 
 #include <string.h>
 
+#include "DialogBox.h"
+
 
 // Number of modifiable colors
 #define COLORS_NUM 9
@@ -37,7 +39,7 @@ struct Theme
     FXbool operator != (const Theme&);
 };
 
-class PreferencesBox : public FXDialogBox
+class PreferencesBox : public DialogBox
 {
     FXDECLARE(PreferencesBox)
 private:
@@ -52,6 +54,8 @@ private:
     FXTextField *txtviewer;
     FXTextField *imgviewer;
     FXTextField *xterm;
+    FXTextField *imgeditor;
+    FXTextField *archiver;
     FXTextField *normalfont;
     FXTextField *textfont;
     FXString    oldiconpath;
@@ -61,6 +65,8 @@ private:
     FXString    oldxterm;
     FXString    oldnormalfont;
     FXString    oldtextfont;
+    FXString    oldimgeditor;
+    FXString    oldarchiver;
     FXCheckButton  *autosave;
     FXCheckButton  *diropen;
     FXCheckButton  *fileopen;
@@ -76,9 +82,10 @@ private:
 	FXCheckButton  *show_mount;
     Theme          Themes[THEMES_NUM];
     Theme          currTheme;
+	FXbool		   use_sudo;
 public:
     enum{
-        ID_ACCEPT=FXDialogBox::ID_LAST,
+        ID_ACCEPT=DialogBox::ID_LAST,
         ID_BROWSE_TXTEDIT,
         ID_BROWSE_TXTVIEW,
 		ID_BROWSE_IMGVIEW,
@@ -89,6 +96,9 @@ public:
         ID_THEME,
         ID_BROWSE_ICON_PATH,
 		ID_TRASH_BYPASS,
+		ID_SU_CMD,
+		ID_SUDO_CMD,
+		ID_WHEELADJUST,
         ID_LAST
     };
 
@@ -102,6 +112,11 @@ public:
     long onCmdBrowsePath(FXObject*,FXSelector,void*);
 	long onCmdNormalFont(FXObject*,FXSelector,void*);
 	long onCmdTextFont(FXObject*,FXSelector,void*);
-	long onUpdTrashBypass(FXObject* o,FXSelector,void*);
+	long onUpdTrashBypass(FXObject*,FXSelector,void*);
+	long onCmdSuMode(FXObject*,FXSelector,void*);
+	long onUpdSuMode(FXObject*,FXSelector,void*);
+	long onCmdWheelAdjust(FXObject*,FXSelector,void*);
+	long onUpdWheelAdjust(FXObject* sender,FXSelector,void*);
+
 };
 #endif
