@@ -1,4 +1,5 @@
 // Properties box
+// Properties box
 
 #include "config.h"
 #include "i18n.h"
@@ -23,6 +24,7 @@
 
 #include "xfedefs.h"
 #include "icons.h"
+#include "DialogBox.h"
 #include "FileDialog.h"
 #include "OverwriteBox.h"
 #include "File.h"
@@ -133,10 +135,10 @@ FXDEFMAP(PropertiesBox) PropertiesBoxMap[]={
         };
 
 // Object implementation
-FXIMPLEMENT(PropertiesBox,FXDialogBox,PropertiesBoxMap,ARRAYNUMBER(PropertiesBoxMap))
+FXIMPLEMENT(PropertiesBox,DialogBox,PropertiesBoxMap,ARRAYNUMBER(PropertiesBoxMap))
 
 // Construct window for one file
-PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER)
+PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): DialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER|DECOR_STRETCHABLE)
 {
     unsigned long long filesize, dirsize=0;
     FXString mod,changed,accessed;
@@ -146,7 +148,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
     FXString type, extension, fileassoc;
     FXbool isLink, isBrokenLink, isDirectory;
     FXString filename;
-    char mnttype[64], used[64], avail[64], size[64];
+    char mnttype[64], used[64], avail[64], pctr[64], size[64];
     char buf[MAXPATHLEN+1];
 	FXString hsize;
 	FXbool ismountpoint=FALSE;
@@ -194,16 +196,16 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
     fassoctab->disable();
 
     new FXLabel(matrix,_("Extension:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    ext = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    ext = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
     new FXLabel(matrix,"",NULL,0);
 
     new FXLabel(matrix,_("Description:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    descr = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    descr = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
     new FXLabel(matrix,"",NULL,0);
 
     new FXLabel(matrix,_("Open:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    open = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_OPEN,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    open = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_OPEN,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 
     int is_ar = FALSE;
 
@@ -211,10 +213,9 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
     FXString editlbl=_("Edit:");
 
 	extension=file.rafter('.',1);
-	if (comparecase(extension,"gz")==0 || comparecase(extension,"tgz")==0 ||
-		comparecase(extension,"tar")==0 || comparecase(extension,"bz2")==0 ||
-		comparecase(extension,"tbz2")==0 || comparecase(extension,"zip")==0 ||
-		comparecase(extension,"Z")==0)
+	extension=extension.lower();
+	if (extension=="gz" || extension=="tgz" || extension=="tar" || extension=="bz2" ||
+		extension=="tbz2" || extension=="zip" || extension=="Z" || extension=="lzh" || extension=="rar")
 	{
 		is_ar = TRUE; // archive
 		viewlbl = _("Extract:");
@@ -224,25 +225,25 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 		editlbl = _("Install/Upgrade:");
 #endif	
     new FXLabel(matrix,viewlbl,NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    view = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_VIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    view = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_VIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 
     if(!is_ar)
     {
         new FXLabel(matrix,editlbl,NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-        edit = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-        new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_EDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+        edit = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+        new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BROWSE_EDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
     }
     else
         edit = NULL;
 
     new FXLabel(matrix,_("Big Icon:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    bigic = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BIG_ICON,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    bigic = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_BIG_ICON,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 
     new FXLabel(matrix,_("Mini Icon:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    miniic = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_MINI_ICON,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    miniic = new FXTextField(matrix,30,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
+    new FXButton(matrix,_("\tSelect file..."),filedialogicon,this,ID_MINI_ICON,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 
     // Label
     new FXLabel(generalframe,_("Name"),NULL,JUSTIFY_LEFT);
@@ -252,7 +253,11 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 	filename=path+PATHSEPSTRING+file;
 	parentdir=path;
 	
-    // Get file/link stat info
+	// Warn if non UTF-8 file name
+	if (!isUtf8(filename.text(),filename.length()))
+		new FXLabel(generalframe,_("=> Warning: file name is not UTF-8 encoded!"),NULL,LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW);
+
+	// Get file/link stat info
     if(lstatrep(filename.text(),&linfo)!=0)
         return ;
 
@@ -277,8 +282,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 			isBrokenLink=TRUE;
 
 		// Get the name of the linked file
-		if (!isBrokenLink)
-			filename=readlink(filename);
+		filename=readlink(filename);
 	}
 
 	// Obtain stat info on the file itself
@@ -334,7 +338,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 		// If it is a mount point
 		if(ismountpoint)
 		{
-			snprintf(buf,sizeof(buf)-1,"df --block-size=1 '%s'",filename.text());
+			// Caution : use the -P option to be POSIX compatible!
+			snprintf(buf,sizeof(buf)-1,"df -P --block-size=1 '%s'",filename.text());
 			p=popen(buf,"r");
 			fgets(buf,sizeof(buf),p);
 			fgets(buf,sizeof(buf),p);
@@ -345,13 +350,15 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 			strlcpy(used,pstr,strlen(pstr)+1);// get used
 			pstr=strtok(NULL," ");
 			strlcpy(avail,pstr,strlen(pstr)+1);// get available
+			pstr=strtok(NULL," ");
+			strlcpy(pctr,pstr,strlen(pstr)+1);// get percentage
 			pclose(p);
 		}
 			
 		// If it is a folder
 		else
 		{
-            type=_("Folder");
+			type=_("Folder");
     		getApp()->beginWaitCursor();
 			strlcpy(buf,dupath.text(),dupath.length()+1);
 			dirsize=pathsize(buf);
@@ -374,8 +381,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
     {
 		// Try to use association table	
 		extension=FXPath::name(filename).rafter('.',1);
-        //if (extension != "")
-			fileassoc=getApp()->reg().readStringEntry("FILETYPES",extension.text(),"");
+		extension=extension.lower();
+		fileassoc=getApp()->reg().readStringEntry("FILETYPES",extension.text(),"");
 
 		// If we have an association
         if(!fileassoc.empty())
@@ -464,6 +471,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
   		mtFree=new FXLabel(mtmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
   		new FXLabel(mtmatrix,_("File system:"),NULL,LAYOUT_LEFT);
   		mtType=new FXLabel(mtmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+  		new FXLabel(mtmatrix,_("Location:"),NULL,LAYOUT_LEFT);
+  		location=new FXLabel(mtmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
 	}
 	else
 	{
@@ -473,8 +482,19 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
   		fileType=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
   		sizelabel=new FXLabel(attrmatrix,_("Total size:"),NULL,LAYOUT_LEFT);
   		fileSize=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
-
-  		FXGroupBox *timegroup=new FXGroupBox(generalframe,_("File Time"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
+  		new FXLabel(attrmatrix,_("Location:"),NULL,LAYOUT_LEFT);
+  		location=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+		if (isLink & !isBrokenLink)
+		{
+			new FXLabel(attrmatrix,_("Link to:"),NULL,LAYOUT_LEFT);
+			linkto=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+		}
+		else if(isBrokenLink)
+		{
+			new FXLabel(attrmatrix,_("Broken link to:"),NULL,LAYOUT_LEFT);
+			linkto=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+		}
+ 		FXGroupBox *timegroup=new FXGroupBox(generalframe,_("File Time"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
   		FXMatrix *timematrix=new FXMatrix(timegroup,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y);
   		new FXLabel(timematrix,_("Last Modified:"),NULL,LAYOUT_LEFT);
   		fileModified=new FXLabel(timematrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
@@ -487,6 +507,13 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
     // File or mount type
 	fileType->setText(type.text());
 
+	// Parent directory
+	location->setText(path);
+	
+	// Refered file for valid or broken link
+	if (isLink)
+		linkto->setText(filename);
+		
 	// If directory
 	if (isDirectory)
 	{
@@ -494,7 +521,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 		if(ismountpoint)
 		{
 			hsize=::hSize(used);
-			mtUsed->setText(hsize);
+        	snprintf(size,sizeof(size)-1,"%s (%s)",hsize.text(),pctr);        
+			mtUsed->setText(size);
 			hsize=::hSize(avail);
 			mtFree->setText(hsize);
 			mtType->setText(mnttype);
@@ -531,7 +559,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 }
 
 // Construct window for multiple files
-PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path): FXDialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER)
+PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path): DialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER|DECOR_STRETCHABLE)
 {
 	unsigned long long filesize, dirsize, totalsize=0;
 	struct stat info;
@@ -552,7 +580,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path):
     ok->addHotKey(KEY_Return);
 
     // Cancel
-    new FXButton(buttons,_("&Cancel"),NULL,this,FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
+    new FXButton(buttons,_("&Cancel"),NULL,this,DialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,20,20);
 
     // Switcher
     FXTabBook *tabbook = new FXTabBook(contents,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);
@@ -568,6 +596,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path):
 	FXLabel *filesType=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
 	new FXLabel(attrmatrix,_("Total size:"),NULL,LAYOUT_LEFT);
 	FXLabel *filesSize=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+	new FXLabel(attrmatrix,_("Location:"),NULL,LAYOUT_LEFT);
+	location=new FXLabel(attrmatrix,FXString::null,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
 
     // Second item is Access Permissions
     new FXTabItem(tabbook,_("&Permissions"),NULL);
@@ -577,7 +607,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path):
 	// This is used as a guess for the username, group and permissions of the whole list
     if(lstatrep(file[1].text(),&info)!=0)
         return ;
-
+	
 	// Obtain user name
 	usrid=FXSystem::userName(info.st_uid);
 
@@ -610,6 +640,9 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path):
     source = "";
     num = n;
 
+	// Parent directory
+	location->setText(path);
+
 	// Start wait cursor
 	getApp()->beginWaitCursor();
 
@@ -646,8 +679,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path):
 
 			// Try to use association table	
 			extension=FXPath::name(files[i]).rafter('.',1);
-       		if (extension != "")
-				fileassoc=getApp()->reg().readStringEntry("FILETYPES",extension.text(),"");
+			extension=extension.lower();
+			fileassoc=getApp()->reg().readStringEntry("FILETYPES",extension.text(),"");
 
 			// Keep the first encountered type
 			if (firstfile)
@@ -706,8 +739,8 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path):
 // Make window
 void PropertiesBox::create()
 {
-    FXDialogBox::create();
-    show(PLACEMENT_SCREEN);
+    DialogBox::create();
+    //show(PLACEMENT_SCREEN);
 }
 
 
@@ -717,7 +750,7 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
 	File *f=NULL;
 	char file[MAXPATHLEN];
 
-    FXDialogBox::onCmdAccept(o,s,p);
+    DialogBox::onCmdAccept(o,s,p);
     if(mode)
     {
 		f = new File(this,_("File permissions"),CHMOD);
@@ -921,7 +954,7 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 	}
 
     // Change perm
-    FXDialogBox::onCmdAccept(o,s,p);
+    DialogBox::onCmdAccept(o,s,p);
 	if (input->getText()==".." || input->getText()==".")
 	{
 		input->setText(source);
@@ -957,7 +990,7 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 
 		// Wait cursor
 		getApp()->beginWaitCursor();
-
+		
 		// Perform chmod on the selected file or directory
 		rc=f->chmod(strdup(filepath.text()),file,m,perm->rec->getCheck(),perm->dironly->getCheck(),perm->fileonly->getCheck());
 
@@ -978,7 +1011,7 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
             MessageBox::error(this,BOX_OK,_("Error"),_("Chmod in %s failed: %s"), file, strerror(errno));
     }
 
-    // chown if needed
+    // Chown if needed
     rc = 0;
     if (oldgrp != perm->grp->getText() || oldusr != perm->user->getText() || perm->rec->getCheck())
     {
@@ -1035,7 +1068,7 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 
 long PropertiesBox::onCmdCancel(FXObject* o,FXSelector s,void* p)
 {
-    FXDialogBox::onCmdCancel(o,s,p);
+    DialogBox::onCmdCancel(o,s,p);
 	
 	// Redraw the file lists
 	((XFileExplorer*) mainWindow)->rpanel->list->recalc();
@@ -1102,7 +1135,7 @@ long PropertiesBox::onCmdCheck(FXObject* o,FXSelector s,void* p)
 
 long PropertiesBox::onCmdBrowse(FXObject* o,FXSelector s,void* p)
 {
-    FileDialog browseProgram(this,_("Select Program"),0,0,0,600,400);
+    FileDialog browseProgram(this,_("Select Program"),0,0,0,650,480);
     const FXchar *patterns[]=
         {
             _("All files"),     "*",NULL
@@ -1145,7 +1178,7 @@ long PropertiesBox::onCmdBrowseIcon(FXObject* o,FXSelector s,void* p)
 			_("GIF Images"),     "*.gif",
             _("BMP Images"),     "*.bmp",NULL
         };
-    FileDialog browseIcon(this,_("Select Icon"),0,0,0,600,400);
+    FileDialog browseIcon(this,_("Select Icon"),0,0,0,650,480);
     browseIcon.setFilename(iconpath+PATHSEPSTRING+icon);
     browseIcon.setPatternList(patterns);
     if(browseIcon.execute())
