@@ -17,6 +17,7 @@
 
 #include "xfedefs.h"
 #include "icons.h"
+#include "FontDialog.h"
 #include "InputDialog.h"
 #include "FileDialog.h"
 #include "MessageBox.h"
@@ -34,6 +35,7 @@ FXDEFMAP(ViewWindow) ViewWindowMap[]={
                                        FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_FIND_AGAIN,ViewWindow::onCmdFind),
                                        FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_QUIT,ViewWindow::onCmdQuit),
 	                                   FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_PRINT,ViewWindow::onCmdPrint),
+									   FXMAPFUNC(SEL_COMMAND,ViewWindow::ID_FONT,ViewWindow::onCmdFont),
                                        FXMAPFUNC(SEL_UPDATE,ViewWindow::ID_FIND_AGAIN,ViewWindow::onCmdFind),
                                        FXMAPFUNC(SEL_UPDATE,ViewWindow::ID_FIND,ViewWindow::onUpdMenu),
                                        FXMAPFUNC(SEL_UPDATE,ViewWindow::ID_PRINT,ViewWindow::onUpdMenu),
@@ -49,23 +51,30 @@ FXIMPLEMENT(ViewWindow,FXMainWindow,ViewWindowMap,ARRAYNUMBER(ViewWindowMap))
 // Make some windows
 ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileView",NULL,NULL,DECOR_ALL,0,0,800,600,0,0),mrufiles(a)
 {
+    FXHotKey hotkey;
+    FXButton *btn = NULL;
+
     // Add to list of windows
     getApp()->windowlist.append(this);
 	
 	setIcon(xfvicon);
 	//setMiniIcon(xfvsmallicon);
 	
-	// Site where to dock
-	FXDockSite *docksite=new FXDockSite(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
+    // Default font
+    font=NULL;
 
     // Make menu bar
-  	FXToolBarShell* dragshell=new FXToolBarShell(this,FRAME_RAISED);
-    menubar=new FXMenuBar(docksite,dragshell,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
-	new FXToolBarGrip(menubar,menubar,FXMenuBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
+    menubar=new FXMenuBar(this,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+
+	// Sites where to dock
+	FXDockSite* topdock=new FXDockSite(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
+	FXDockSite* bottomdock=new FXDockSite(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
+	FXDockSite* leftdock=new FXDockSite(this,LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
+	FXDockSite* rightdock=new FXDockSite(this,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y);
 
     // Tool bar
   	FXToolBarShell* dragshell1=new FXToolBarShell(this,FRAME_RAISED);
-  	toolbar=new FXToolBar(docksite,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
   	new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
     int style=BUTTON_TOOLBAR;
 
@@ -77,7 +86,11 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     filemenu=new FXMenuPane(this);
     new FXMenuTitle(menubar,_("&File"),NULL,filemenu);
 
-    // Search Menu
+    // Edit menu
+    editmenu=new FXMenuPane(this);
+    new FXMenuTitle(menubar,_("&Edit"),NULL,editmenu);
+
+	// Search Menu
     searchmenu=new FXMenuPane(this);
     new FXMenuTitle(menubar,_("&Search"),NULL,searchmenu);
 
@@ -117,16 +130,22 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     // Toolbar button: Print
 	new FXButton(toolbar,_("\tPrint document (Ctrl-P)\tPrint document file. (Ctrl-P)"),printicon,this,ID_PRINT,style|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
-    // Toolbar buttons: Searching
+	// Toolbar button: Quit
+    btn=new FXButton(toolbar,_("\tQuit Xfv (Ctrl-Q)\tQuit Xfv. (Ctrl-Q)"),quiticon,this,ID_QUIT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+    btn->hide();
+    hotkey=(CONTROLMASK<<16) | KEY_w;
+    btn->addHotKey(hotkey);
+
+	// Toolbar button: Copy
+    new FXSeparator(toolbar,SEPARATOR_GROOVE);
+    new FXButton(toolbar,_("\tCopy (Ctrl-C)\tCopy selection to clipboard. (Ctrl-C)"),copy_clpicon,viewer,FXText::ID_COPY_SEL,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
+
+	// Toolbar buttons: Searching
     new FXSeparator(toolbar,SEPARATOR_GROOVE);
 	//new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,10);
     new FXButton(toolbar,_("\tFind a string (Ctrl-F)\tFind string in document. (Ctrl-F)"),searchicon,this,ID_FIND,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
     new FXButton(toolbar,_("\tFind again (Ctrl-G)\tFind again. (Ctrl-G)"),find_againicon,this,ID_FIND_AGAIN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
 
-	// Toolbar button: Quit
-    new FXSeparator(toolbar,SEPARATOR_GROOVE);
-    new FXButton(toolbar,_("\tQuit Xfv (Ctrl-Q)\tQuit Xfv. (Ctrl-Q)"),quiticon,this,ID_QUIT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
-
     // File Menu entries
     new FXMenuCommand(filemenu,_("&Open file...\tCtrl-O\tOpen document file. (Ctrl-O)"),fileopenicon,this,ID_OPEN);
     new FXMenuCommand(filemenu,_("&Print file...\tCtrl-P\tPrint document file. (Ctrl-P)"),printicon,this,ID_PRINT);
@@ -147,13 +166,18 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
 
 	new FXMenuCommand(filemenu,_("&Quit\tCtrl-Q\tQuit Xfv. (Ctrl Q)"),quiticon,this,ID_QUIT);
 
-    // Search Menu entries
+	// Edit menu
+	new FXMenuCommand(editmenu,_("&Copy\tCtrl-C\tCopy selection to clipboard. (Ctrl-C)"),copy_clpicon,viewer,FXText::ID_COPY_SEL);
+
+	// Search Menu entries
     new FXMenuCommand(searchmenu,_("&Find a string...\tCtrl-F\tFind a string in a document. (Ctrl-F)"),searchicon,this,ID_FIND);
     new FXMenuCommand(searchmenu,_("Find &again\tCtrl-G\tFind again. (Ctrl-G)"),find_againicon,this,ID_FIND_AGAIN);
 
+
     // Preferences menu
     new FXMenuCheck(prefsmenu,_("&Toolbar\t\tDisplay or hide toolbar."),toolbar,FXWindow::ID_TOGGLESHOWN);
     new FXMenuCheck(prefsmenu,_("&Status bar\t\tDisplay or hide status bar."),statusbar,FXWindow::ID_TOGGLESHOWN);
+    new FXMenuCommand(prefsmenu,_("&Font...\t\tChange text font."),fontsicon,this,ID_FONT);
 
     // Window menu
     new FXMenuRadio(windowmenu,FXString::null,this,ID_WINDOW_1);
@@ -213,7 +237,13 @@ ViewWindow::ViewWindow(XFileView* a,const FXString& file):FXMainWindow(a,"XFileV
     // Make a tool tip
     new FXToolTip(getApp(),0);
 
-    // Recent files
+	// Dialogs
+	printdialog=NULL;
+	opendialog=NULL;
+	finddialog=NULL;
+	findtext=NULL;
+
+	// Recent files
     mrufiles.setTarget(this);
     mrufiles.setSelector(ID_OPEN_RECENT);
 
@@ -232,10 +262,11 @@ void ViewWindow::create()
 	
 	// Set Xfv text font according to the Xfe registry
 	FXString fontspec;
-	fontspec=reg_xfe->readStringEntry("SETTINGS","textfont","Helvetica,100,normal,regular");
+	fontspec=getApp()->reg().readStringEntry("SETTINGS","textfont","Helvetica,100,normal,regular");
 	if(!fontspec.empty())
 	{
     	FXFont* font=new FXFont(getApp(),fontspec);
+		font->create();
         viewer->setFont(font);
 	}
 	delete reg_xfe;
@@ -245,10 +276,10 @@ void ViewWindow::create()
 	getApp()->setWheelLines(wheellines);
 	
     // Get size and position
-    FXuint width=getApp()->reg().readUnsignedEntry("SETTINGS","width",600);
-    FXuint height=getApp()->reg().readUnsignedEntry("SETTINGS","height",400);
-    FXint xpos=getApp()->reg().readIntEntry("SETTINGS","xpos",0);
-    FXint ypos=getApp()->reg().readIntEntry("SETTINGS","ypos",0);
+    FXuint ww=getApp()->reg().readUnsignedEntry("SETTINGS","width",800);
+    FXuint hh=getApp()->reg().readUnsignedEntry("SETTINGS","height",600);
+    FXint xx=getApp()->reg().readIntEntry("SETTINGS","xpos",0);
+    FXint yy=getApp()->reg().readIntEntry("SETTINGS","ypos",0);
 
 	// Get toolbar status
     if(getApp()->reg().readUnsignedEntry("SETTINGS","showtoolbar",TRUE)==FALSE)
@@ -258,11 +289,9 @@ void ViewWindow::create()
     if(getApp()->reg().readUnsignedEntry("SETTINGS","showstatusbar",TRUE)==FALSE)
         statusbar->hide();
    
-	// Reposition window
-    position(xpos,ypos,width,height);
-
     FXMainWindow::create();
     filemenu->create();
+	editmenu->create();
     searchmenu->create();
     prefsmenu->create();
     windowmenu->create();
@@ -270,6 +299,9 @@ void ViewWindow::create()
     show(PLACEMENT_DEFAULT);
     viewer->setFocus();
     viewer->handle(this,FXSEL(SEL_COMMAND,FXText::ID_TOGGLE_EDITABLE),NULL);
+
+	// Reposition window
+	position(xx,yy,ww,hh);
 }
 
 
@@ -280,12 +312,18 @@ ViewWindow::~ViewWindow()
     delete menubar;
     delete toolbar;
 	delete statusbar;
+	delete font;
 	delete filemenu;
+	delete editmenu;
 	delete searchmenu;
 	delete prefsmenu;
 	delete helpmenu;
  	delete windowmenu;
     delete contents;
+	delete printdialog;
+	delete opendialog;
+	delete finddialog;
+	delete findtext;
 }
 
 
@@ -366,13 +404,15 @@ long ViewWindow::onCmdOpen(FXObject*,FXSelector,void*)
             _("HTML Files"),         "*.htm",
             _("PHP Files"),          "*.php",NULL
         };
-    FileDialog opendialog(this,_("Open Document"),0,0,0,600,400);
-	opendialog.setSelectMode(SELECTFILE_MULTIPLE);
-    opendialog.setPatternList(patterns);
-  	opendialog.setDirectory(FXPath::directory(filename));
-    if(opendialog.execute())
+    
+	if (opendialog==NULL)
+		opendialog=new FileDialog(this,_("Open Document"),0,0,0,650,480);
+	opendialog->setSelectMode(SELECTFILE_MULTIPLE);
+    opendialog->setPatternList(patterns);
+  	opendialog->setDirectory(FXPath::directory(filename));
+    if(opendialog->execute())
     {
-        FXString* files=opendialog.getFilenames();
+        FXString* files=opendialog->getFilenames();
 		FXuint i=0;
 		while (files[i]!=FXString::null)
 		{
@@ -427,13 +467,12 @@ long ViewWindow::onCmdPrint(FXObject*,FXSelector,void*)
 	
 	// Open print dialog filled with the current print command
 	int rc=1;
-	InputDialog* dialog;
-	dialog=new InputDialog(this,printcommand,_("Enter print command: \n(ex: lpr -P <printer>)"),_("Print"),"",printbigicon);    
-	dialog->setText(printcommand);
-	dialog->CursorEnd();
-    rc=dialog->execute(PLACEMENT_CURSOR);
-	printcommand=dialog->getText();
-	delete dialog;
+	if (printdialog==NULL)
+		printdialog=new InputDialog(this,printcommand,_("Print command: \n(ex: lpr -P <printer>)"),_("Print"),"",printbigicon);    
+	printdialog->setText(printcommand);
+	printdialog->CursorEnd();
+    rc=printdialog->execute(PLACEMENT_CURSOR);
+	printcommand=printdialog->getText();
 	
 	// If cancel was pressed, exit
 	if (!rc)
@@ -450,15 +489,38 @@ long ViewWindow::onCmdPrint(FXObject*,FXSelector,void*)
 }
 
 
+// Change text font
+long ViewWindow::onCmdFont(FXObject*,FXSelector,void*)
+{
+    FontDialog fontdlg(this,_("Change Font"),DECOR_BORDER|DECOR_TITLE);
+    FXFontDesc fontdesc;
+    viewer->getFont()->getFontDesc(fontdesc);
+    fontdlg.setFontSelection(fontdesc);
+    if(fontdlg.execute())
+    {
+        FXFont *oldfont=font;
+        fontdlg.getFontSelection(fontdesc);
+        font=new FXFont(getApp(),fontdesc);
+        font->create();
+        viewer->setFont(font);
+        delete oldfont;
+    }
+	saveConfig();
+    return 1;
+}
+
+
 // Save configuration when quitting
 void ViewWindow::saveConfig()
 {
-    // Write new window size and position back to registry
-	getApp()->reg().writeUnsignedEntry("SETTINGS","width",(FXuint)getWidth());
-	getApp()->reg().writeUnsignedEntry("SETTINGS","height",(FXuint)getHeight());
-	getApp()->reg().writeIntEntry("SETTINGS","xpos",getX());
-	getApp()->reg().writeIntEntry("SETTINGS","ypos",getY());
+    FXString fontspec;
 
+	// Write new window size and position back to registry
+    getApp()->reg().writeUnsignedEntry("SETTINGS","width",(FXuint)getWidth());
+    getApp()->reg().writeUnsignedEntry("SETTINGS","height",(FXuint)getHeight());
+    getApp()->reg().writeIntEntry("SETTINGS","xpos",getX());
+    getApp()->reg().writeIntEntry("SETTINGS","ypos",getY());
+	
     // Toolbar status
     if(toolbar->shown())
         getApp()->reg().writeUnsignedEntry("SETTINGS","showtoolbar",TRUE);
@@ -471,6 +533,10 @@ void ViewWindow::saveConfig()
     else
         getApp()->reg().writeUnsignedEntry("SETTINGS","showstatusbar",FALSE);
 
+    // Font
+    fontspec=viewer->getFont()->getFont();
+    getApp()->reg().writeStringEntry("SETTINGS","textfont",fontspec.text());
+
 	// Write registry settings
 	getApp()->reg().write();
 }
@@ -530,24 +596,23 @@ long ViewWindow::onCmdFind(FXObject* obj,FXSelector sel,void* ptr)
 
     if(FXSELID(sel)==ID_FIND)
     {
-        FXDialogBox dialog(this,_("Find"),DECOR_TITLE|DECOR_BORDER);
-        FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
-        FXTextField* text=new FXTextField(content,30,NULL,0,FRAME_SUNKEN|LAYOUT_FILL_X);
-        new FXHorizontalSeparator(content,SEPARATOR_GROOVE|LAYOUT_FILL_X);
-        FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
-        new FXButton(buttons,_("&Cancel"),NULL,&dialog,FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
-        new FXButton(buttons,_("&Find"),NULL,&dialog,FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
-        text->setText(str);
-        text->setSelection(0,str.length());
-        text->setFocus();
-
-        if(!dialog.execute())
+        if (finddialog==NULL)
 		{
-			delete text;
-            return 0;
+			finddialog=new DialogBox(this,_("Find"),DECOR_TITLE|DECOR_BORDER);
+			FXVerticalFrame* content=new FXVerticalFrame(finddialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
+			findtext=new FXTextField(content,30,NULL,0,FRAME_SUNKEN|LAYOUT_FILL_X);
+			new FXHorizontalSeparator(content,SEPARATOR_GROOVE|LAYOUT_FILL_X);
+			FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
+			new FXButton(buttons,_("&Cancel"),NULL,finddialog,DialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
+			new FXButton(buttons,_("&Find"),NULL,finddialog,DialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
 		}
-        str=text->getText();
-		delete text;
+		findtext->setText(str);
+		findtext->setSelection(0,str.length());
+		findtext->setFocus();
+
+        if(!finddialog->execute())
+            return 0;
+        str=findtext->getText();
     }
     loc=find(str,viewer->getCursorPos());
     if(loc==-1)
@@ -578,6 +643,9 @@ FXint ViewWindow::find(const FXString str,FXint pos)
 // Close window
 FXbool ViewWindow::close(FXbool notify)
 {
+	// Save settings 
+    saveConfig();
+
     return FXMainWindow::close(notify);
 }
 
