@@ -15,9 +15,13 @@
 #include "FileDict.h"
 #include "FilePanel.h"
 #include "InputDialog.h"
+#include "HistInputDialog.h"
+#include "BrowseInputDialog.h"
 #include "Properties.h"
 #include "DirPanel.h"
 #include "Bookmarks.h"
+#include "Preferences.h"
+#include "TextWindow.h"
 
 
 // Helper function
@@ -65,7 +69,8 @@ protected:
     FXMenuTitle		*rpanelmenutitle;
     FXMenuTitle		*helpmenutitle;
     Bookmarks		*bookmarks;
-    FXToolBar		*toolbar;
+    FXToolBar		*generaltoolbar;
+    FXToolBar		*paneltoolbar;
     FXToolBar		*locationbar;
 	FXComboBox		*address;
     FXStatusBar		*status;
@@ -88,6 +93,10 @@ protected:
     FXButton		*btntrash;
 	FXArrowButton   *btnbackhist;
 	FXArrowButton   *btnforwardhist;
+	HistInputDialog*   rundialog;
+	PreferencesBox*    prefsdialog;
+	TextWindow*		   helpwindow;
+	FXString message;
 public:
     enum{
         ID_ABOUT=FXMainWindow::ID_LAST,
@@ -99,11 +108,13 @@ public:
 		ID_DIR_FORWARD,
 		ID_DIR_BACK_HIST,
 		ID_DIR_FORWARD_HIST,
+		ID_FILE_PROPERTIES,
         ID_FILE_COPY,
         ID_FILE_RENAME,
         ID_FILE_MOVE,
         ID_FILE_COPYTO,
         ID_FILE_CUT,
+		ID_FILE_PASTE,
         ID_FILE_SYMLINK,
         ID_FILE_DELETE,
         ID_FILE_TRASH,
@@ -152,7 +163,6 @@ public:
 	long onCmdSu(FXObject*,FXSelector,void*);
     long onCmdXTerm(FXObject*,FXSelector,void*);
 	long onCmdEmptyTrashcan(FXObject*,FXSelector,void*);
-    long onCmdFileMan(FXObject*,FXSelector,void*);
 	long onCmdShowPanels(FXObject*,FXSelector,void*);
     long onCmdRestart(FXObject*,FXSelector,void*);
 	long onCmdNewWindow(FXObject*,FXSelector,void*);
@@ -175,5 +185,16 @@ public:
 	long onUpdDirBackHist(FXObject*,FXSelector,void*);
 	long onCmdDirForwardHist(FXObject*,FXSelector,void*);
 	long onUpdDirForwardHist(FXObject*,FXSelector,void*);
+	long onCmdFileCopyClp(FXObject*,FXSelector,void*);
+	long onCmdFileCutClp(FXObject*,FXSelector,void*);
+	long onCmdFilePasteClp(FXObject*,FXSelector,void*);
+	long onCmdFileRename(FXObject*,FXSelector,void*);
+	long onCmdFileMove(FXObject*,FXSelector,void*);
+	long onCmdFileCopyto(FXObject*,FXSelector,void*);
+	long onCmdFileSymlink(FXObject*,FXSelector,void*);
+	long onUpdFileMan(FXObject*,FXSelector,void*);
+	long onUpdFilePaste(FXObject*,FXSelector,void*);
+	long onCmdFileProperties(FXObject*,FXSelector,void*);
+	long onUpdFileRename(FXObject*,FXSelector,void*);
 };
 #endif
