@@ -155,6 +155,9 @@ FXDEFMAP(XFileImage) XFileImageMap[]={
                                			FXMAPFUNC(SEL_UPDATE,XFileImage::ID_LABEL,XFileImage::onUpdLabel),
                                         FXMAPFUNC(SEL_COMMAND,XFileImage::ID_TOGGLE_FIT_WIN,XFileImage::onCmdToggleFitWin),
 										FXMAPFUNC(SEL_UPDATE,XFileImage::ID_TOGGLE_FIT_WIN,XFileImage::onUpdToggleFitWin),
+										FXMAPFUNC(SEL_COMMAND,XFileImage::ID_HOME,XFileImage::onCmdHome),
+										FXMAPFUNC(SEL_COMMAND,XFileImage::ID_WORK,XFileImage::onCmdWork),
+										FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,XFileImage::ID_FILELIST,XFileImage::onCmdPopupMenu),
                                     };
 
 
@@ -173,17 +176,18 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL,0,0,8
     setTarget(this);
     setSelector(ID_TITLE);
 
-	// Site where to dock
-	FXDockSite *docksite=new FXDockSite(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
-
     // Make menu bar
-  	FXToolBarShell* dragshell=new FXToolBarShell(this,FRAME_RAISED);
-    menubar=new FXMenuBar(docksite,dragshell,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
-	new FXToolBarGrip(menubar,menubar,FXMenuBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
+    menubar=new FXMenuBar(this,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+
+	// Sites where to dock
+	FXDockSite* topdock=new FXDockSite(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
+	FXDockSite* bottomdock=new FXDockSite(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
+	FXDockSite* leftdock=new FXDockSite(this,LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
+	FXDockSite* rightdock=new FXDockSite(this,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y);
 
     // Tool bar
   	FXToolBarShell* dragshell1=new FXToolBarShell(this,FRAME_RAISED);
-  	toolbar=new FXToolBar(docksite,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
+  	toolbar=new FXToolBar(topdock,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
   	new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
 
     // File menu
@@ -219,7 +223,7 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL,0,0,8
     filebox=new FXHorizontalFrame(splitter,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
 
     // Make file list
-    FXVerticalFrame* fileframe=new FXVerticalFrame(filebox,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
+    FXVerticalFrame* fileframe=new FXVerticalFrame(filebox,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
     label=new FXLabel(fileframe," ",NULL,JUSTIFY_LEFT|LAYOUT_LEFT|LAYOUT_FILL_X|FRAME_RAISED);
     label->setTarget(this);
     label->setSelector(FXSEL(0,XFileImage::ID_LABEL));
@@ -248,19 +252,22 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL,0,0,8
 	new FXButton(toolbar,_("\tPrint image (Ctrl-P)\tPrint image file. (Ctrl-P)"),printicon,this,ID_PRINT,BUTTON_TOOLBAR|FRAME_RAISED);
 
     // Toolbar button: Quit
-    new FXButton(toolbar,_("\tQuit Xfi (Ctrl-Q)\tQuit Xfi. (Ctrl-Q)"),quiticon,this,ID_QUIT,BUTTON_TOOLBAR|FRAME_RAISED);
+    btn=new FXButton(toolbar,_("\tQuit Xfi (Ctrl-Q)\tQuit Xfi. (Ctrl-Q)"),quiticon,this,ID_QUIT,BUTTON_TOOLBAR|FRAME_RAISED);
+    btn->hide();
+    hotkey=(CONTROLMASK<<16) | KEY_w;
+    btn->addHotKey(hotkey);
 
 	// Separator
 	toolbarSeparator(toolbar);
 
     // Toolbar button: Zoom in
     btn=new FXButton(toolbar,_("\tZoom in (Ctrl+)\tZoom in image. (Ctrl+)"),zoominicon,this,ID_ZOOM_IN,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
-    hotkey=(0004<<16) |  KEY_KP_Add;
+    hotkey=(CONTROLMASK<<16) |  KEY_KP_Add;
     btn->addHotKey(hotkey);
 
     // Toolbar button: Zoom out
     btn=new FXButton(toolbar,_("\tZoom out (Ctrl-)\tZoom out image. (Ctrl-)"),zoomouticon,this,ID_ZOOM_OUT,ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
-    hotkey=(0004<<16) | KEY_KP_Subtract;
+    hotkey=(CONTROLMASK<<16) | KEY_KP_Subtract;
     btn->addHotKey(hotkey);
 
     // Toolbar button: Zoom 100%
@@ -313,7 +320,7 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL,0,0,8
     new FXMenuCommand(imagemenu,_("Zoom &in\tCtrl+\tZoom in image. (Ctrl+)"),zoominicon,this,ID_ZOOM_IN);
     new FXMenuCommand(imagemenu,_("Zoom &out\tCtrl-\tZoom out image. (Ctrl-)"),zoomouticon,this,ID_ZOOM_OUT);
     new FXMenuCommand(imagemenu,_("Zoo&m 100%\tCtrl-I\tZoom image to 100%. (Ctrl-I)"),zoom100icon,this,ID_ZOOM_100);
-    new FXMenuCommand(imagemenu,_("Zoom to fit &window\tCtrl-W\tZoomto fit window. (Ctrl-W)"),zoomwinicon,this,ID_ZOOM_WIN);
+    new FXMenuCommand(imagemenu,_("Zoom to fit &window\tCtrl-F\tZoomto fit window. (Ctrl-F)"),zoomwinicon,this,ID_ZOOM_WIN);
     new FXMenuCommand(imagemenu,_("Rotate &right\tCtrl-R\tRotate right. (Ctrl-R)"),rotaterighticon,this,ID_ROTATE_270);
     new FXMenuCommand(imagemenu,_("Rotate &left\tCtrl-L\tRotate left. (Ctrl-L)"),rotatelefticon,this,ID_ROTATE_90);
     new FXMenuCommand(imagemenu,_("Mirror &horizontally\tCtrl-H\tMirror horizontally. (Ctrl-H)"),fliplricon,this,ID_MIRROR_HOR);
@@ -340,6 +347,10 @@ XFileImage::XFileImage(FXApp* a):FXMainWindow(a,"Xfi ",NULL,NULL,DECOR_ALL,0,0,8
 
     // Make a tool tip
     new FXToolTip(getApp(),TOOLTIP_NORMAL);
+	
+	// Dialogs
+	printdialog=NULL;
+	opendialog=NULL;
 
     // Recent files
     mrufiles.setTarget(this);
@@ -373,12 +384,68 @@ XFileImage::~XFileImage()
     delete prefsmenu;
     delete viewmenu;
     delete dragshell1;
-    delete dragshell2;
 	delete label;
 	delete filelist;
 	delete img;
 	delete tmpimg;
-	
+	delete printdialog;
+	delete opendialog;
+}
+
+
+long XFileImage::onCmdPopupMenu(FXObject* o,FXSelector s,void* p)
+{
+    // Popup menu pane
+	FXMenuPane menu(this);
+    FXint x,y;
+    FXuint state;
+    getRoot()->getCursorPosition(x,y,state);
+
+	new FXMenuCommand(&menu,_("Go ho&me"),homeicon,this,ID_HOME);
+	new FXMenuCommand(&menu,_("Go &work"),workicon,this,ID_WORK);
+	new FXMenuSeparator(&menu);
+	new FXMenuCheck(&menu,_("&Hidden files"),filelist,FileList::ID_TOGGLE_HIDDEN);
+	new FXMenuCheck(&menu,_("Thum&bnails"),filelist,FileList::ID_TOGGLE_THUMBNAILS);
+	new FXMenuSeparator(&menu);
+	new FXMenuRadio(&menu,_("B&ig icons"),filelist,FXIconList::ID_SHOW_BIG_ICONS);
+	new FXMenuRadio(&menu,_("&Small icons"),filelist,FXIconList::ID_SHOW_MINI_ICONS);
+	new FXMenuRadio(&menu,_("F&ull file list"),filelist,FXIconList::ID_SHOW_DETAILS);
+	new FXMenuSeparator(&menu);
+	new FXMenuRadio(&menu,_("&Rows"),filelist,FileList::ID_ARRANGE_BY_ROWS);
+	new FXMenuRadio(&menu,_("&Columns"),filelist,FileList::ID_ARRANGE_BY_COLUMNS);
+	new FXMenuSeparator(&menu);
+	new FXMenuRadio(&menu,_("&Name"),filelist,FileList::ID_SORT_BY_NAME);
+	new FXMenuRadio(&menu,_("Si&ze"),filelist,FileList::ID_SORT_BY_SIZE);
+	new FXMenuRadio(&menu,_("&Type"),filelist,FileList::ID_SORT_BY_TYPE);
+	new FXMenuRadio(&menu,_("E&xtension"),filelist,FileList::ID_SORT_BY_EXT);
+	new FXMenuRadio(&menu,_("&Date"),filelist,FileList::ID_SORT_BY_TIME);
+	new FXMenuSeparator(&menu);
+	new FXMenuCheck(&menu,_("I&gnore case"),filelist,FileList::ID_SORT_CASE);
+	new FXMenuCheck(&menu,_("Dir&ectories first"),filelist,FileList::ID_DIRS_FIRST);
+	new FXMenuCheck(&menu,_("Re&verse order"),filelist,FileList::ID_SORT_REVERSE);
+
+	menu.create();
+	allowPopupScroll=TRUE;  // Allow keyboard scrolling
+	menu.popup(NULL,x,y);
+	getApp()->runModalWhileShown(&menu);
+	allowPopupScroll=FALSE;
+	return 1;
+}
+
+
+// Back to home directory
+long XFileImage::onCmdHome(FXObject*,FXSelector,void*)
+{
+    filelist->setDirectory(FXSystem::getHomeDirectory());
+    return 1;
+}
+
+
+// Back to current working directory
+long XFileImage::onCmdWork(FXObject*,FXSelector,void*)
+{
+    filelist->setDirectory(FXSystem::getCurrentDirectory());
+    return 1;
 }
 
 
@@ -541,17 +608,17 @@ long XFileImage::onUpdToggleFitWin(FXObject* sender,FXSelector,void*)
 }
 
 
-
 // Open
 long XFileImage::onCmdOpen(FXObject*,FXSelector,void*)
 {
-    FileDialog open(this,_("Open Image"),0,0,0,600,400);
-	open.setSelectMode(SELECTFILE_EXISTING);
-    open.setPatternList(patterns);
-  	open.setDirectory(filelist->getDirectory());
-    if(open.execute())
+    if (opendialog==NULL)
+		opendialog=new FileDialog(this,_("Open Image"),0,0,0,650,480);
+	opendialog->setSelectMode(SELECTFILE_EXISTING);
+    opendialog->setPatternList(patterns);
+  	opendialog->setDirectory(filelist->getDirectory());
+    if(opendialog->execute())
     {
-        filename=open.getFilename();
+        filename=opendialog->getFilename();
         filelist->setCurrentFile(filename);
         mrufiles.appendFile(filename);
         loadimage(filename);
@@ -569,13 +636,12 @@ long XFileImage::onCmdPrint(FXObject*,FXSelector,void*)
 	
 	// Open print dialog filled with the current print command
 	int rc=1;
-	InputDialog* dialog;
-	dialog=new InputDialog(this,printcommand,_("Enter print command: \n(ex: lpr -P <printer>)"),_("Print"),"",printbigicon);    
-	dialog->setText(printcommand);
-	dialog->CursorEnd();
-    rc=dialog->execute(PLACEMENT_CURSOR);
-	printcommand=dialog->getText();
-	delete dialog;
+	if (printdialog==NULL)
+		printdialog=new InputDialog(this,printcommand,_("Print command: \n(ex: lpr -P <printer>)"),_("Print"),"",printbigicon);    
+	printdialog->setText(printcommand);
+	printdialog->CursorEnd();
+    rc=printdialog->execute(PLACEMENT_CURSOR);
+	printcommand=printdialog->getText();
 	
 	// If cancel was pressed, exit
 	if (!rc)
@@ -1083,8 +1149,8 @@ void XFileImage::start(FXString startimage)
 void XFileImage::create()
 {
 	// Get size and position
-    FXuint ww=getApp()->reg().readUnsignedEntry("SETTINGS","width",600);
-    FXuint hh=getApp()->reg().readUnsignedEntry("SETTINGS","height",400);
+    FXuint ww=getApp()->reg().readUnsignedEntry("SETTINGS","width",800);   // Workaround for a possible bug in some WMs
+    FXuint hh=getApp()->reg().readUnsignedEntry("SETTINGS","height",600);  // Workaround for a possible bug in some WMs
     FXint xx=getApp()->reg().readIntEntry("SETTINGS","xpos",0);
     FXint yy=getApp()->reg().readIntEntry("SETTINGS","ypos",0);
 
@@ -1253,6 +1319,7 @@ int main(int argc,char *argv[])
 	if(!fontspec.empty())
 	{
     	FXFont* normalFont=new FXFont(application,fontspec);
+		normalFont->create();
         application->setNormalFont(normalFont);
     }
 
@@ -1294,6 +1361,10 @@ int main(int argc,char *argv[])
 	if (!loadicons)
 		 MessageBox::error(application,BOX_OK,_("Error loading icons"),_("Unable to load some icons. Please check your icons path!"));
 
+	// Tooltips setup time and duration
+	application->setTooltipPause(TOOLTIP_PAUSE);
+	application->setTooltipTime(TOOLTIP_TIME);
+
     // Start
     window->start(startimage);
 
