@@ -22,6 +22,7 @@
 #include "FileDialog.h"
 #include "MessageBox.h"
 #include "ViewWindow.h"
+#include "File.h"
 #include "XFileView.h"
 
 // Add FOX hacks
@@ -192,6 +193,7 @@ int main(int argc,char *argv[])
     selforecolor=reg->readColorEntry("SETTINGS","selforecolor",FXRGB(255,255,255));
     selbackcolor=reg->readColorEntry("SETTINGS","selbackcolor",FXRGB(10,36,106));
     bordercolor=reg->readColorEntry("SETTINGS","bordercolor",FXRGB(0,0,0));
+	highlightcolor=reg->readColorEntry("SETTINGS","highlightcolor",FXRGB(238,238,238));
 	application->setBaseColor(basecolor);
 	application->setForeColor(forecolor);
 	application->setBackColor(backcolor);
@@ -214,6 +216,10 @@ int main(int argc,char *argv[])
 	if (!loadicons)
 		 MessageBox::error(application,BOX_OK,_("Error loading icons"),_("Unable to load some icons. Please check your icons path!"));
 
+	// Tooltips setup time and duration
+	application->setTooltipPause(TOOLTIP_PAUSE);
+	application->setTooltipTime(TOOLTIP_TIME);
+
     // Loop over args
     for(i=1; i<argc; ++i)
     {
