@@ -19,11 +19,14 @@
 #include "File.h"
 #include "icons.h"
 
+FXMainWindow *mainWindow;
+
+
 // Add FOX hacks
 #include "foxhacks.cc"
 
 // Global variables
-FXMainWindow *mainWindow;
+//FXMainWindow *mainWindow;
 FXString startdir="";
 char **args;
 FXbool iconic=FALSE;
@@ -104,10 +107,10 @@ int main(int argc,char *argv[])
 
 #if defined(linux)
 
-    // Test the existence of apt-get to see if the Linux distro is Debian based
-	// This is done by checking the existence of the string 'apt'
-	// in the response to the command 'apt-get --version
-	FXString cmd="apt-get --version 2>&1";
+    // Test the existence of dpkg to see if the Linux distro is Debian based
+	// This is done by checking the existence of the string 'Debian'
+	// in the response to the command 'dpkg' (with --version argument)
+	FXString cmd="dpkg --version 2>&1";
     FILE *debcmd=popen(cmd.text(),"r");
     if(!debcmd)
     {
@@ -119,7 +122,7 @@ int main(int argc,char *argv[])
     while(fgets(textdeb,sizeof(textdeb),debcmd))
         bufdeb+=textdeb;
     snprintf(textdeb,sizeof(textdeb)-1,"%s",bufdeb.text());
-	if (strstr(textdeb,"apt")!=NULL)
+	if (strstr(textdeb,"Debian")!=NULL)
 		deb_based=TRUE;
 	pclose(debcmd);
 	
@@ -215,6 +218,11 @@ int main(int argc,char *argv[])
 	application->addSignal(SIGINT,mainWindow,XFileExplorer::ID_QUIT);
 
 	application->create();
+	
+	// Tooltips setup time and duration
+	application->setTooltipPause(TOOLTIP_PAUSE);
+	application->setTooltipTime(TOOLTIP_TIME);
+
 	if (!loadicons)
 		 MessageBox::error(application,BOX_OK,_("Error loading icons"),_("Unable to load some icons. Please check your icons path!"));
    	application->run();
