@@ -50,7 +50,15 @@
 #define DEFAULTICONPATH   "~/.xfe/icons/xfe-theme:/usr/local/share/xfe/icons/xfe-theme:/usr/share/xfe/icons/xfe-theme"
 #endif
 
-// Command to launch Xfe as root
-#ifndef SUCMD
-#define SUCMD " -fn 7x14 -geometry 60x4 -bc +sb -e su -c xfe"
+// Command to launch Xfe as root using sudo or su, using an xterm
+#ifndef SUDOCMD
+#define SUDOCMD "-fn 7x14 -geometry 60x4 -bc +sb -e sudo su -c xfe"
 #endif
+
+#ifndef SUCMD
+#define SUCMD "-fn 7x14 -geometry 60x4 -bc +sb -e su -c xfe"
+#endif
+
+// Tooltips setup time and duration
+#define TOOLTIP_PAUSE 200
+#define TOOLTIP_TIME 10000
